package com.kuxuan.autoclick.net.retrofit;

import android.util.Log;

import com.google.gson.Gson;
import com.kuxuan.autoclick.app.Mapp;
import com.kuxuan.autoclick.model.Constant;
import com.kuxuan.autoclick.net.interceptor.CustomLogInterceptor;
import com.kuxuan.autoclick.net.api.APIService;
import com.kuxuan.autoclick.net.callback.ResultCallBack;
import com.kuxuan.autoclick.utils.DecodeUttils;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.SystemUtils;

import java.io.IOException;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

/**
 * @Auth lkj
 * @Date 2023/6/27 16:26
 * @Describe Retrofit网络请求工具类，这是通用的
 */
public class RetrofitUtils {
    //1.声明api
    private APIService apiService;

    //2.实现单例模式
    static class SingleHolder {
        public static RetrofitUtils instance = new RetrofitUtils();
    }

    public static RetrofitUtils getInstance() {
        return SingleHolder.instance;
    }

    //3.构造器
    private RetrofitUtils() {
        String baseUrl = null;
        String deviceid = SystemUtils.getDeviceId(Mapp.context);
        Set<String> set = new HashSet<String>(Arrays.asList(Constant.testDevicesIds));
        if (set.contains(deviceid)) {
            LogUtil.e("这是测试手机的设备id");
//            baseUrl = Constant.http_debug;
            baseUrl = Constant.http_release;
        } else {
            LogUtil.e("这是正式手机的设备id");
            baseUrl = Constant.http_release;
        }

        //1.初始化okhttp 并配置OKhttp
        OkHttpClient okHttpClient = new OkHttpClient.Builder()
                .connectTimeout(10, TimeUnit.SECONDS)
                .readTimeout(10, TimeUnit.SECONDS)
                .writeTimeout(10, TimeUnit.SECONDS)
                .addInterceptor(CustomLogInterceptor.getInstance())//添加日志拦截器
                .addInterceptor(new Interceptor() {
                    @Override
                    public Response intercept(Chain chain) throws IOException {
                        Request request = chain.request().newBuilder().build();
                        return chain.proceed(request);
                    }
                })
                .build();
        //初始化retrofit 并且配置retrofit
        Retrofit retrofit = new Retrofit.Builder()
                .baseUrl(baseUrl)
                .addConverterFactory(GsonConverterFactory.create(new Gson()))
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                .client(okHttpClient)
                .build();
        apiService = retrofit.create(APIService.class);

    }

    //4.创建日志拦截器
    private HttpLoggingInterceptor getHttpLoggingInterceptor() {
        HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor(new HttpLoggingInterceptor.Logger() {
            @Override
            public void log(String message) {
                String result = DecodeUttils.decodeUnicode(message);
                LogUtil.e("requestLog   " + result);
            }
        });
        httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
        return httpLoggingInterceptor;
    }

    //5.将APIService向外提供
    public APIService getApiService() {
        return apiService;
    }

    //6.写一个通用的 获取数据的公共方法
    public <T> void getDataFromAPI(Observable<T> observable, final ResultCallBack callBack) {
        //1.绑定生命周期
//        callBack.autoDispose(observable);
        //2.使用观察者观察数据
        observable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<T>() {
                    @Override
                    public void onSubscribe(Disposable d) {
//                        callBack.startRequest();
                    }

                    @Override
                    public void onNext(T t) {
                        callBack.success(t);
                    }

                    @Override
                    public void onError(Throwable e) {
                        Log.e("TAG", "onError: " + e.getCause() + "\n" + e.getMessage());
                        if (e instanceof ConnectException) {
                            callBack.fail("网络中断！！请检查你的网络");
                        } else if (e instanceof SocketTimeoutException) {
                            callBack.fail("网络中断！！请检查你的网络");
                        } else {
                            callBack.fail("不知道啥异常");
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void onComplete() {
                        callBack.requestComplete();
                    }
                });
    }
}
