package com.kuxuan.autoclick.net.updateapp.manager;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.content.FileProvider;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.kuxuan.autoclick.BuildConfig;
import com.kuxuan.autoclick.R;
import com.kuxuan.autoclick.model.ApkConfigBean;
import com.kuxuan.autoclick.model.HisToryBean;
import com.kuxuan.autoclick.ui.MainActivity;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.MyX509TrustManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.SecureRandom;
import java.util.List;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

public class AutoUpdater {
    // 下载安装包的网络路径
    private String apkUrl = "https://playlet.oss-cn-beijing.aliyuncs.com/android_tool/";
    protected String checkUrl = apkUrl + "output.json";
    protected String hisApkUrl = apkUrl + "his_apks.txt";
    protected String update_content = apkUrl + "update_content.txt";

    // 保存APK的文件名
//    private static String saveFileName = "autoClick.apk";
    private static File apkFile;

    // 下载线程
    private Thread downLoadThread;
    private int progress;// 当前进度
    // 是否是最新的应用,默认为false
    private boolean isNew = false;
    private boolean intercept = false;
    // 进度条与通知UI刷新的handler和msg常量
    private ProgressBar mProgress;
    private TextView txtStatus;


    private static final int NORMAL = 0;
    private static final int DOWN_UPDATE = 1;
    private static final int DOWN_OVER = 2;
    private static final int SHOWDOWN = 3;
    private String onLineApkName;
    private AlertDialog.Builder downLoadDialog;
    private TextView insertBtn;
    private AlertDialog alertDialog;

    private static AutoUpdater autoUpdater;
    private static Activity mContext;
    private int typeA;
    private boolean isOnResume;
    private View redDot;

    public static AutoUpdater getInstance(Activity context) {
        mContext = context;
        if (autoUpdater == null) {
            synchronized (AutoUpdater.class) {
                if (autoUpdater == null) {
                    autoUpdater = new AutoUpdater();
                }
            }
        }
        return autoUpdater;
    }

    public void setRedDot(View redDot) {
        this.redDot = redDot;
    }

    /**
     * @author lkj
     * @date 2024/3/4 14:03
     * @desc 提示需要安装apk的权限
     */
    public void showPermissionDialog(Activity activity) {
        AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        builder.setCancelable(false);
        builder.setTitle("提示");
        builder.setMessage("群发工具未获取安装apk的权限！");
        builder.setPositiveButton("去授权", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                //跳转到该应用的安装应用的权限页面
                Uri packageURI = Uri.parse("package:" + BuildConfig.APPLICATION_ID);
                //注意这个是8.0新API
                Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI);
                activity.startActivityForResult(intent, MainActivity.INSTALL_APK_REQUEST_CODE);
            }
        });
        builder.create().show();
    }

    public void ShowUpdateDialog(String message, String onLineApkName, int type) {
        typeA = type;
        this.onLineApkName = onLineApkName;
        apkFile = new File(mContext.getApplicationContext().getExternalFilesDir(null), onLineApkName);
        AlertDialog.Builder builder = new AlertDialog.Builder(mContext);
        builder.setCancelable(false);
        builder.setTitle("软件版本更新");
        builder.setMessage(message);
        builder.setPositiveButton("立即下载", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                DownloadApk();
                ShowDownloadDialog();
            }
        });
        builder.setNegativeButton("以后再说", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        builder.create().show();
    }

    private void ShowDownloadDialog() {
        downLoadDialog = new AlertDialog.Builder(mContext);
        downLoadDialog.setCancelable(false);
        downLoadDialog.setTitle("软件版本更新");
        LayoutInflater inflater = LayoutInflater.from(mContext);
        View v = inflater.inflate(R.layout.progress, null);
        mProgress = (ProgressBar) v.findViewById(R.id.progress);
        txtStatus = v.findViewById(R.id.txtStatus);
        insertBtn = v.findViewById(R.id.insert_btn);
        downLoadDialog.setView(v);
        alertDialog = downLoadDialog.show();
        insertBtn.setOnClickListener(view -> {
            intercept = true;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                PackageManager pm = mContext.getPackageManager();
                //pm.canRequestPackageInstalls() 返回用户是否授予了安装apk的权限
                if (pm.canRequestPackageInstalls()) {
                    alertDialog.dismiss();
                    installAPK();
                } else {
                    showPermissionDialog(mContext);
                }
            }
        });
    }

    /**
     * 检查是否更新的内容
     *
     * @param isOnResume
     */
    public void CheckUpdate(boolean isOnResume) {
        this.isOnResume = isOnResume;
        new Thread(new Runnable() {
            @Override
            public void run() {
                int version_code = 0;
                try {
                    version_code = mContext.getPackageManager().getPackageInfo(mContext.getPackageName(), 0).versionCode;
                } catch (PackageManager.NameNotFoundException e) {
                    e.printStackTrace();
                }
                String versionName = "1";
                String outputFile = "";
                String config = doGet(checkUrl);
                String updateContent = doGet(update_content);
                if (TextUtils.isEmpty(config)) {
                    mHandler.sendEmptyMessage(NORMAL);
                    LogUtil.e("服务器上没有发现apk的配置文件！！");
                    return;
                }
                LogUtil.e("apk-config: " + config);
                Type listType = new TypeToken<List<ApkConfigBean>>() {
                }.getType();
                List<ApkConfigBean> apkConfigBeans = new Gson().fromJson(config, listType);
                boolean isHaveNewVersion = false;
                for (ApkConfigBean apkConfigBean : apkConfigBeans) {
                    int versionCode = apkConfigBean.getApkData().getVersionCode();
                    onLineApkName = apkConfigBean.getApkData().getOutputFile();
                    String versionName1 = apkConfigBean.getApkData().getVersionName();
                    if (version_code < versionCode) {
                        apkFile = new File(mContext.getApplicationContext().getExternalFilesDir(null), onLineApkName);
                        if (!apkFile.exists()) {
                            LogUtil.e("文件不存在，重新创建！");
                            try {
                                if (apkFile.createNewFile()) {
                                    // 文件创建成功
                                    LogUtil.e("文件创建成功");
                                } else {
                                    // 文件创建失败
                                    LogUtil.e("文件创建失败");
                                }
                            } catch (IOException e) {
                                // 处理异常
                                e.printStackTrace();
                            }
                        } else {
                            LogUtil.e("文件存在，重新创建！" + apkFile.getPath());
                        }
                        isHaveNewVersion = true;
                        apkUrl = apkUrl + outputFile;
                        Message message = Message.obtain();
                        message.what = SHOWDOWN;
                        message.obj = "发现群发工具新版本：" + versionName1 + ", 是否下载？\n" + updateContent;
                        mHandler.sendMessage(message);
                        break;
                    }
                }
                if (!isHaveNewVersion) {
                    mHandler.sendEmptyMessage(NORMAL);
                }
            }
        }).start();
    }

    public void CheckHisApks(Handler handler) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                String config = doGet(hisApkUrl);
                if (TextUtils.isEmpty(config)) {
                    LogUtil.e("服务器上没有发现apk的配置文件！！");
                    return;
                }
                HisToryBean hisToryBean = new Gson().fromJson(config, HisToryBean.class);
                Message obtain = Message.obtain();
                obtain.what = 111;
                obtain.obj = hisToryBean;
                handler.sendMessage(obtain);
            }
        }).start();
    }

    /**
     * 从服务器下载APK安装包
     */
    public void DownloadApk() {
        downLoadThread = new Thread(DownApkWork);
        downLoadThread.start();
    }

    private Runnable DownApkWork = new Runnable() {
        @Override
        public void run() {
            URL url;
            try {
                //如果下载地址是HTTPS，则把这段加上，http则不需要
                SSLContext sslContext = SSLContext.getInstance("SSL");//第一个参数为 返回实现指定安全套接字协议的SSLContext对象。第二个为提供者
                TrustManager[] tm = {new MyX509TrustManager()};
                sslContext.init(null, tm, new SecureRandom());
                SSLSocketFactory ssf = sslContext.getSocketFactory();


//                url = new URL(apkUrl + onLineApkName);
                url = new URL(apkUrl + onLineApkName);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.connect();
                int length = conn.getContentLength();
                InputStream ins = conn.getInputStream();
                LogUtil.e("apk存放地址：" + apkFile);
                FileOutputStream fos = new FileOutputStream(apkFile);
                int count = 0;
                byte[] buf = new byte[1024];
                while (!intercept) {
                    int numread = ins.read(buf);
                    count += numread;
                    progress = (int) (((float) count / length) * 100);
                    // 下载进度
                    mHandler.sendEmptyMessage(DOWN_UPDATE);
                    if (numread <= 0) {
                        // 下载完成通知安装
                        mHandler.sendEmptyMessage(DOWN_OVER);
                        break;
                    }
                    fos.write(buf, 0, numread);
                }
                fos.close();
                ins.close();


            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * 安装APK内容
     */
    public void installAPK() {
        try {
            if (!apkFile.exists()) {
                LogUtil.e("apk不存在！");
                return;
            }
            LogUtil.e("判断如何安装apk");
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);//安装完成后打开新版本
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION); // 给目标应用一个临时授权
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {//判断版本大于等于7.0
                //如果SDK版本>=24，即：Build.VERSION.SDK_INT >= 24，使用FileProvider兼容安装apk
                LogUtil.e("SDK >= 24安装apk");
                String packageName = mContext.getApplicationContext().getPackageName();
                String authority = new StringBuilder(packageName).append(".fileprovider").toString();
                LogUtil.e("authority: " + authority);
                Uri apkUri = FileProvider.getUriForFile(mContext, authority, apkFile);
                intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
            } else {
                LogUtil.e("SDK ＜ 24安装apk");
                intent.setDataAndType(Uri.fromFile(apkFile), "application/vnd.android.package-archive");
            }
            mContext.startActivity(intent);
            android.os.Process.killProcess(android.os.Process.myPid());//安装完之后会提示”完成” “打开”。


        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @SuppressLint("HandlerLeak")
    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case NORMAL:
                    if (!isOnResume) {
                        Toast.makeText(mContext, "当前已是最新版本", Toast.LENGTH_SHORT).show();
                    } else {
                        redDot.setVisibility(View.GONE);
                    }
                    break;
                case SHOWDOWN:
                    if (!isOnResume) {
                        ShowUpdateDialog((String) msg.obj, onLineApkName, 1);
                    } else {
                        redDot.setVisibility(View.VISIBLE);
                    }
                    break;
                case DOWN_UPDATE:
                    txtStatus.setText(progress + "%");
                    mProgress.setProgress(progress);
                    break;
                case DOWN_OVER:
                    txtStatus.setText(100 + "%");
                    mProgress.setProgress(100);
                    if (typeA == 1) {
                        insertBtn.setVisibility(View.VISIBLE);
                        Toast.makeText(mContext, "下载完毕", Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(mContext, "请到文件夹中查看文件", Toast.LENGTH_SHORT).show();
                        alertDialog.dismiss();
                    }
                    break;
                default:
                    break;
            }
        }
    };

    public static String doGet(String httpurl) {
        HttpURLConnection connection = null;
        InputStream is = null;
        BufferedReader br = null;
        String result = null;
        try {
            URL url = new URL(httpurl);
            connection = (HttpURLConnection) url.openConnection();
            connection.setRequestMethod("GET");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);
            connection.connect();
            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                StringBuffer sbf = new StringBuffer();
                String temp = null;
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\r\n");
                }
                result = sbf.toString();
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            connection.disconnect();
        }
        return result;
    }
}