package com.kuxuan.autoclick.service;


import android.annotation.SuppressLint;
import android.app.Service;
import android.content.Intent;
import android.graphics.PixelFormat;
import android.graphics.drawable.AnimationDrawable;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.provider.Settings;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.kuxuan.autoclick.R;
import com.kuxuan.autoclick.adapter.TextAdapter;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.MemoruyUtils;
import com.kuxuan.autoclick.utils.NetWorkUtils;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

public class FloatingWindowService extends Service {
    public static boolean isStarted = false;

    private WindowManager windowManager;
    private WindowManager.LayoutParams layoutParams;

    private View displayView;

    public static final int DMS_INPUT_IMG_W = 16;
    public static final int DMS_INPUT_IMG_H = 16;
    private ImageView redLight;
    private AnimationDrawable anim;

    //有可能会内存泄漏,采用虚引用
    private WeakReference<FloatingWindowService> myService = new WeakReference<>(FloatingWindowService.this);
    private Work work;
    private TextView msgTv, msgTv2, msgTv3, msgTv4, msgTv5;
    private RecyclerView recyclerView;
    private TextAdapter textAdapter;
    private List<String> strings;
//    private TextView textTip1;

    @Override
    public boolean onUnbind(Intent intent) {
        LogUtil.e("接触绑定服务！ ");
        return super.onUnbind(intent);
    }

    @Override
    public IBinder onBind(Intent intent) {
        LogUtil.e("绑定服务");
        showFloatingWindowVideo();
        return getWork();//new MyBinder();
    }

    @Override
    public void onCreate() {
        super.onCreate();
        isStarted = true;
        windowManager = (WindowManager) getSystemService(WINDOW_SERVICE);
        layoutParams = new WindowManager.LayoutParams();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            layoutParams.type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY; //TYPE_APPLICATION_SUB_PANEL ; //TYPE_APPLICATION_OVERLAY;
        } else {
            layoutParams.type = WindowManager.LayoutParams.TYPE_PHONE;
        }
        layoutParams.format = PixelFormat.RGBA_8888;
        layoutParams.gravity = Gravity.LEFT | Gravity.TOP;
        layoutParams.flags = WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL | WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE | WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE;
        int width = windowManager.getDefaultDisplay().getWidth();
        layoutParams.width = (width - 10 * 2) / 2 + 20;
//        layoutParams.width = WindowManager.LayoutParams.MATCH_PARENT;
//        layoutParams.height = DMS_INPUT_IMG_H;
        layoutParams.x = 10;
        layoutParams.y = 0;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return super.onStartCommand(intent, flags, startId);
    }

    @SuppressLint("NewApi")
    private void showFloatingWindowVideo() {
        if (Settings.canDrawOverlays(this)) {
            LayoutInflater layoutInflater = LayoutInflater.from(this);
            displayView = layoutInflater.inflate(R.layout.window_display, null);
            displayView.setOnTouchListener(new FloatingOnTouchListener());
            redLight = displayView.findViewById(R.id.green_light);
            msgTv3 = displayView.findViewById(R.id.message3);
            msgTv4 = displayView.findViewById(R.id.message4);
            msgTv5 = displayView.findViewById(R.id.message5);
            msgTv = displayView.findViewById(R.id.message1);
            msgTv2 = displayView.findViewById(R.id.message2);
//            textTip1 = displayView.findViewById(R.id.text_tip1);
            recyclerView = displayView.findViewById(R.id.recycle_view);
            int height = msgTv.getLayoutParams().height;
            layoutParams.height = height;
            windowManager.addView(displayView, layoutParams);
            initAdapter();
        }
    }

    private void initAdapter() {
        LinearLayoutManager manager = new LinearLayoutManager(this);
        manager.setOrientation(RecyclerView.VERTICAL);
        strings = new ArrayList<>();
        textAdapter = new TextAdapter(this, strings);
        recyclerView.setLayoutManager(manager);
        recyclerView.setAdapter(textAdapter);
    }

    public void startGreenPoint() {
        anim = (AnimationDrawable) getResources().getDrawable(R.drawable.green_light_anim);
        redLight.setBackground(anim);
        anim.start();
    }

    public void stopGreenPoint() {
        anim.stop();
        redLight.setBackground(getResources().getDrawable(R.drawable.red_light));
    }

    @Override
    public void onDestroy() {
        stopSelf();
        super.onDestroy();
    }

    private class FloatingOnTouchListener implements View.OnTouchListener {
        private int x;
        private int y;

        @Override
        public boolean onTouch(View view, MotionEvent event) {
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    x = (int) event.getRawX();
                    y = (int) event.getRawY();
                    break;
                case MotionEvent.ACTION_MOVE:
                    int nowX = (int) event.getRawX();
                    int nowY = (int) event.getRawY();
                    int movedX = nowX - x;
                    int movedY = nowY - y;
                    x = nowX;
                    y = nowY;
                    layoutParams.x = layoutParams.x + movedX;
                    layoutParams.y = layoutParams.y + movedY;
                    windowManager.updateViewLayout(view, layoutParams);
                    break;
                default:
                    break;
            }
            return true;
        }
    }

    private Work getWork() {
        if (work == null) {
            work = new Work();
        }
        return work;
    }

    public class Work extends Binder {
        public void startWork() {
            //开始工作
            startGreenPoint();
        }

        public void stopWork() {
            //停止工作
            stopGreenPoint();
        }

        public FloatingWindowService getMyService() {
            return myService.get();
        }

        public void setData(String msg, int what, int arg1) {
            Message obtain = Message.obtain();
            obtain.what = what;
            obtain.obj = msg;
            obtain.arg1 = arg1;
            handler.sendMessage(obtain);
        }
    }

    private Handler handler = new Handler(new Handler.Callback() {
        @SuppressLint("SetTextI18n")
        @Override
        public boolean handleMessage(@NonNull Message msg) {
            if (msg.what == 40) {
                msgTv3.setText((String) msg.obj);
//                textTip1.setText((String) msg.obj);
//                textTip1.setTextColor(getResources().getColor(R.color.green));
                LogUtil.e((String) msg.obj);
                int index = msg.arg1;
                if (index > 0) {
                    new Thread(new Runnable() {
                        @Override
                        public void run() {
                            for (int i = index; i > -1; i--) {
                                Message obtain = Message.obtain();
                                obtain.what = 40;
                                if (i == 0) {
//                                    obtain.obj = NetWorkUtils.isNetworkConnected(FloatingVideoService.this) ? "网络正常" : "网络异常";
                                    obtain.obj = "";
                                } else {
                                    obtain.obj = "等待" + i + "s";
                                }
                                obtain.arg1 = 0;
                                handler.sendMessage(obtain);
                                try {
                                    Thread.sleep(1000);
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }).start();
                }
            }
            if (msg.what == 10) {
                msgTv.setText((String) msg.obj);
                boolean connected = NetWorkUtils.isNetworkConnected(FloatingWindowService.this);
                if (connected) {
                    msgTv3.setText("网络正常");
                } else {
                    msgTv3.setText("网络异常");
                }
            }
            if (msg.what == 20) {
                msgTv2.setText((String) msg.obj);
            }
            if (msg.what == 30) {
                if (textAdapter != null) {
                    strings.add((String) msg.obj);
                    if (strings.size() > 5) {
                        strings.remove(0);
                    }
                    textAdapter.addList(strings);
                }
            }
            if (msg.what == 50) {
//                MemoruyUtils.getMemoryUsage("com.tencent.wework");
                msgTv4.setText("网速：" + msg.obj);
//                textTip1.setText("网速：" + msg.obj);
//                textTip1.setTextColor(getResources().getColor(R.color.green));
                int percent = MemoruyUtils.memoryUsageRate(FloatingWindowService.this);
                msgTv5.setText("内存：" + percent + "%");
                if (percent <= 70) {
                    msgTv5.setTextColor(getResources().getColor(R.color.green));
                } else if (percent > 70 && percent <= 90) {
                    msgTv5.setTextColor(getResources().getColor(R.color.yellow));
                } else {
                    msgTv5.setTextColor(getResources().getColor(R.color.red));
                }
            }
            return false;
        }
    });

}

//生命周期上的区别

//执行startService时，Service会经历onCreate->onStartCommand。当执行stopService时，直接调用onDestroy方法。调用者如果没有stopService，Service会一直在后台运行，下次调用者再起来仍然可以stopService。

//执行bindService时，Service会经历onCreate->onBind。这个时候调用者和Service绑定在一起。调用者调用unbindService方法或者调用者Context不存在了（如Activity被finish了），Service就会调用onUnbind->onDestroy。这里所谓的绑定在一起就是说两者共存亡了。
