package com.kuxuan.autoclick.service

import android.annotation.SuppressLint
import android.graphics.Rect
import android.text.TextUtils
import android.view.accessibility.AccessibilityNodeInfo
import com.blankj.utilcode.util.ThreadUtils.runOnUiThread
import com.kuxuan.autoclick.model.Constant
import com.kuxuan.autoclick.model.TaskBean
import com.kuxuan.autoclick.service.WeworkController.weworkService
import com.kuxuan.autoclick.utils.*
import com.kuxuan.autoclick.utils.AccessibilityUtil.scrollByXY_sec
import java.lang.Thread.sleep
import java.text.SimpleDateFormat

/**
 * 当前是否在首页
 */
fun isAtHome(taskBean: TaskBean): Boolean {
    LogUtil.e("判断 当前是否在首页")
    val isLogin = getWindow(0)?.findAccessibilityNodeInfosByText("已阅读并同意 软件许可及服务协议 和 隐私政策")
    if (isLogin != null && isLogin.size > 0) {
        runOnUiThread(Runnable {
            showToast("企业微信已经退出登录，请重新登陆！")
        })
        return true
    }
    LogUtil.e("判断 当前是否登陆")
    val rootCopy = getRootCopy(taskBean)
    LogUtil.e("这里收到返回值 继续下面的判断！！")
    if (rootCopy == null) {
        LogUtil.e("rootCopy 为空 这里被阻断了")
        return false
    }
    val msgLists = AccessibilityUtil.findAllOnceByText(rootCopy, "消息", exact = true)
    LogUtil.e("判断 消息 tab 有没有被选中  " + msgLists.size)
    /**
     * @auth lkj
     * @Date 2023/5/30 10:24
     * @Describe 返回符合条件的第一个元素，没有返回null
     */
    for (msgList in msgLists) {
        if (msgList.text != null) {
            LogUtil.e("获取的消息按钮： " + msgList.text.toString())
        }
        if (msgList.text != null && msgList.text.equals("消息")) {
            val rect = Rect()
            msgList.getBoundsInScreen(rect)
            //左：0   上: 1523   右: 144   下: 1601
            //左：0   上: 1524   右: 144   下: 1607
            LogUtil.e("消息的位置：左：" + rect.left + "   上: " + rect.top + "   右: " + rect.right + "   下: " + rect.bottom)
            if (rect.top == 1523 || rect.top == 1524) {
                if (!msgList.isSelected) {
                    LogUtil.e("点击一下 消息 按钮 ${msgList.isSelected}")
                    AccessibilityUtil.performClick(msgList)
                    return true
                } else {
                    LogUtil.e("消息 按钮 ${msgList.isSelected}")
                    return true
                }
            } else {
                LogUtil.e("消息 按钮的参数变了：" + rect.left + "   上: " + rect.top + "   右: " + rect.right + "   下: " + rect.bottom)
            }
        }
        LogUtil.e("消息 按钮 ${msgList.isSelected}")
    }
    return false
}

/**
 * 获取企业微信窗口
 */
fun getRoot(): AccessibilityNodeInfo {
    return getRoot(false)
}

/**
 * 获取前台窗口
 * @param ignoreCheck false 必须等待前台为企业微信 true 直接返回当前前台窗口
 */
fun getRoot(ignoreCheck: Boolean): AccessibilityNodeInfo {
    while (true) {
        sleep(Constant.CHANGE_PAGE_INTERVAL, 1)
        var root = weworkService.rootInActiveWindow
        for (window in weworkService.windows) {
            if (window.root != null) {
                root = window.root
            } else {
                LogUtil.e("根节点检测： 检测为空")
            }
        }
        if (root != null) {
            LogUtil.e("获取 weworkService.rootInActiveWindow 不为空")
            if (root.packageName == Constant.PACKAGE_NAMES) {
                return root
            } else {
                LogUtil.e("当前不在企业微信: ${root.packageName}")
                //                }
                if (ignoreCheck) {
                    return root
                }
            }
        } else {
            LogUtil.e("获取 weworkService.rootInActiveWindow 为空了")
        }
    }
}

fun getRootCopy(taskBean: TaskBean): AccessibilityNodeInfo? {
    val timeFlag = System.currentTimeMillis()
    var loopGoOn = 0
    while (loopGoOn < 50) {
        loopGoOn++
        if (!taskBean.isLive) {
            val dafor = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
            LogUtil.e("任务已取消：${dafor.format(taskBean.id)}")
            weworkService.taskList.remove(taskBean)
            return null
        }
        val currTime = System.currentTimeMillis()
        if (currTime - timeFlag > 1000 * 60) {
            LogUtil.e("已经执行一分钟了 都找不到企微图标！！")
        }
        try {
            sleep(1000, 1)
        } catch (e: Exception) {
            LogUtil.e("延时2秒报错：${e.printStackTrace()}")
            e.printStackTrace()
        }
        if (!ScreenUtil.isScreenOn(weworkService)) {
            LogUtil.e("判断当前屏幕是黑屏状态，重新唤醒一下")
            ScreenUtil.wakeAndUnlock(weworkService, true)
        }
        /**
         * @auth lkj
         * @Date 2023/6/12 14:44
         * @Describe 设置在息屏状态下  服务也保持运行状态
         */
        var root: AccessibilityNodeInfo? = null
        for (window in weworkService.windows) {
            if (window.root != null) {
                val root1: AccessibilityNodeInfo? = window.root
                if (null != root1) {
//                    LogUtil.e("根节点检测：" + root1.packageName)
                    if (root1?.packageName?.equals(Constant.PACKAGE_NAMES)!! || root1.packageName?.equals("com.miui.home")!!) {
                        root = root1
                    }
                }
            } else {
                val texts = AccessibilityUtil.findAllByClazz(weworkService.rootInActiveWindow, Views.TextView)
                val builder = StringBuilder()
                var count = 0
                for (text in texts) {
                    if (text.text != null) {
                        count++
                        builder.append(count.toString() + ". " + text.text.toString() + "  ")
                    }
                }
                LogUtil.e("可获得的信息：  ${builder.toString()}")
                if (builder.toString().contains("邀请你进行语音通话") || builder.toString().contains("邀请你进行视频通话")) {
                    for (text in texts) {
                        //[52,175][479,219]
                        if (!TextUtils.isEmpty(text.text.toString()) && (text.text.toString().contains("邀请你进行语音通话") || text.text.toString().contains("邀请你进行视频通话"))) {
                            val rect = Rect()
                            text.getBoundsInScreen(rect)
                            if ((rect.top >= 173 && rect.top <= 177) && (rect.bottom >= 217 && rect.bottom <= 221)) {
                                LogUtil.e("上滑挂断电话: " + text.text.toString())
                                hangUpCall()
                            } else {
                                LogUtil.e("发现覆盖页面形式的 来电 ")
                            }
                        }
                    }
                }
            }
        }
        if (root != null) {
//            LogUtil.e("获取 rootInActiveWindow 不为空")
            if (root.packageName == Constant.PACKAGE_NAMES) {
                LogUtil.e("root值: ${root.packageName}")
                return root
            } else {
                LogUtil.e("当前不在企业微信: ${root.packageName}")
                val texts = AccessibilityUtil.findAllByClazz(getWindow(0), Views.TextView)
                val builder = StringBuilder()
                var count = 0
                for (text in texts) {
                    if (text.text != null) {
                        count++
                        builder.append(count.toString() + ". " + text.text.toString() + "  ")
                    }
                }
                LogUtil.e("可获得的信息：  ${builder.toString()}")
                if (builder.toString().contains("邀请你进行语音通话") || builder.toString().contains("邀请你进行视频通话")) {
                    for (text in texts) {
                        //[52,175][479,219]
                        if (!TextUtils.isEmpty(text.text.toString()) && (text.text.toString().contains("邀请你进行语音通话") || text.text.toString().contains("邀请你进行视频通话"))) {
                            val rect = Rect()
                            text.getBoundsInScreen(rect)
                            if ((rect.top >= 173 && rect.top <= 177) && (rect.bottom >= 217 && rect.bottom <= 221)) {
                                LogUtil.e("上滑挂断电话: " + text.text.toString())
                                hangUpCall()
                            } else {
                                LogUtil.e("发现覆盖页面形式的 来电 ")
                            }
                        }
                    }
                }
                if (root.packageName.equals("com.miui.home")) {
                    clickScreenApp(taskBean)
                } else {
                    if (!root.packageName.equals("com.kuxuan.autoclick")) {
                        LogUtil.e("在其他APP中")

                        AccessibilityUtil.scrollByXY(weworkService, 0, 1000, 400, 0)
                        AccessibilityUtil.scrollByXY(weworkService, 0, 1000, 400, 0)
                        sleep(NumberUtils.getRandom1000())
                    } else {
                        LogUtil.e("在群发工具中")
                    }
                }
                if (root.packageName.equals("android")) {

                }
                LogUtil.e("跳过上面的判断！！")
            }
        } else {
            LogUtil.e("获取 rootInActiveWindow 为空了")
            if (!ScreenUtil.isScreenOn(weworkService)) {
                LogUtil.e("判断当前屏幕是黑屏状态，重新唤醒一下")
                ScreenUtil.wakeAndUnlock(weworkService, true)
            }
        }
    }
    return null
}

/**
 * 需要选择工作身份时 选择其中一个进入主界面
 */
@SuppressLint("NewApi")
fun toStatus(taskBean: TaskBean) {
    //需要选择工作身份时 选择其中一个进入主界面
    LogUtil.e("选择工作身份")
    val workStatusList = getWindow(0)?.findAccessibilityNodeInfosByText("选择工作身份")
    if (workStatusList != null) {
        selectStatus(workStatusList, taskBean)
    } else {
        LogUtil.e("任务标识 全部设置 false")
        setFalse()
    }
}

fun selectStatus(workStatusList: List<AccessibilityNodeInfo>, taskBean: TaskBean) {
    //点击一个工作身份进入

    val textViews = AccessibilityUtil.findAllByClazz(getWindow(0), Views.TextView)
    if (textViews != null) {
        if (!textViews.isEmpty() && textViews.size > 0) {
            for (textView in textViews) {
                val rect = Rect()
                textView.getBoundsInScreen(rect)
                LogUtil.e("找的所有的TextView：  ${rect.left}  ${textView.text} ")
                if (rect.left == 32 && !textView.text.equals("选择身份进入")) {
                    val x = (rect.left + rect.right) / 2
                    val y = (rect.top + rect.bottom) / 2
                    AccessibilityUtil.clickByNode(weworkService, x, y)
                    LogUtil.e("点击工作工作身份列表")
                    break
                }
            }
        } else {
            LogUtil.e("textViews 没找打节点！！")
        }
    } else {
        LogUtil.e("textViews 为空了")
    }

    sleep(NumberUtils.getRandom1000() - 2000, 1)
    val goInto = getWindow(0)?.findAccessibilityNodeInfosByText("进入")
    if (goInto != null && !goInto.isEmpty() && goInto.size > 0) {
        //点击一个工作身份进入
        if (!taskBean.isLive) {
            LogUtil.e("任务取消 ${dafor.format(taskBean.id)} ${taskBean.isLive}")
            weworkService.taskList.remove(taskBean)
            return
        }
        AccessibilityUtil.clickByNodeInScreen(weworkService, goInto.get(0))
        LogUtil.e("点击工作身份")
        sleep(NumberUtils.getRandom1000() * 3, 1)
    } else {
        LogUtil.e("任务标识 全部设置 false")
        setFalse()
    }
}

fun hangUpCall() {
    AccessibilityUtil.scrollByXY(weworkService, 360, 250)
    scrollByXY_sec(weworkService, 360, 250, 0, -250)
}