package com.kuxuan.autoclick.service;

import android.graphics.Rect;
import android.os.SystemClock;
import android.text.TextUtils;
import android.view.accessibility.AccessibilityNodeInfo;
import android.view.accessibility.AccessibilityWindowInfo;

import com.kuxuan.autoclick.model.Constant;
import com.kuxuan.autoclick.model.TaskBean;
import com.kuxuan.autoclick.utils.AccessibilityUtil;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.NumberUtils;
import com.kuxuan.autoclick.utils.ScreenUtil;
import com.kuxuan.autoclick.utils.Views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import static com.kuxuan.autoclick.service.GlobalMethodKt.getRootCopy;

/**
 * @author lkj
 * @date 2024/4/24 18:21
 * @desc
 */
public class GlobalMethodJava {
    private static GlobalMethodJava globalMethodJava;

    public static GlobalMethodJava getInstance() {
        if (globalMethodJava == null) {
            synchronized (GlobalMethodJava.class) {
                if (globalMethodJava == null) {
                    globalMethodJava = new GlobalMethodJava();
                }
            }
        }
        return globalMethodJava;
    }

    /**
     * @author lkj
     * @date 2024/4/24 18:31
     * @desc 须等待前台为企业微信 true 直接返回当前前台窗口
     */
    public AccessibilityNodeInfo getRoot(boolean ignoreCheck) {
        while (true) {
            SystemClock.sleep(NumberUtils.getRandom1000());
            AccessibilityNodeInfo nodeInfo = null;
            for (AccessibilityWindowInfo window : WeworkController.INSTANCE.getWeworkService().getWindows()) {
                if (window.getRoot() != null) {
                    nodeInfo = window.getRoot();
                } else {
                    LogUtil.e("根节点检测： 检测为空");
                }
            }
            if (nodeInfo != null) {
                LogUtil.e("获取 weworkService.rootInActiveWindow 不为空");
                if (nodeInfo.getPackageName() == Constant.PACKAGE_NAMES) {
                    return nodeInfo;
                } else {
                    LogUtil.e("当前不在企业微信: ${root.packageName}");
                    //                }
                    if (ignoreCheck) {
                        return nodeInfo;
                    }
                }
            } else {
                LogUtil.e("获取 weworkService.rootInActiveWindow 为空了");
            }
        }
    }

    /**
     * @author lkj
     * @date 2024/4/24 18:27
     * @desc 当前是否在首页
     */
    public boolean isAtHome(TaskBean taskBean) {
        LogUtil.e("判断 当前是否在首页");
        ArrayList<AccessibilityNodeInfo> allByClazz = AccessibilityUtil.INSTANCE.findAllByClazz(getRootCopy(taskBean), new String[]{Views.TextView});
        for (AccessibilityNodeInfo byClazz : allByClazz) {
            if (!TextUtils.isEmpty(byClazz.getText())) {
                String string = byClazz.getText().toString();
                if (string.equals("已阅读并同意 软件许可及服务协议 和 隐私政策")) {
                    return true;
                }
            }
        }
        return false;
    }

    public AccessibilityNodeInfo getRootCopy(TaskBean taskBean) {
        long timeFlag = System.currentTimeMillis();
        int loopGoOn = 0;
        while (loopGoOn < 50) {
            loopGoOn++;
            if (!taskBean.isLive()) {
                SimpleDateFormat dafor = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                LogUtil.e("任务已取消：" + dafor.format(taskBean.getId()));
                WeworkController.weworkService.taskList.remove(taskBean);
                return null;
            }
            long currTime = System.currentTimeMillis();
            if (currTime - timeFlag > 1000 * 60) {
                LogUtil.e("已经执行一分钟了 都找不到企微图标！！");
            }
            try {
                SystemClock.sleep(1000);
            } catch (Exception e) {
                LogUtil.e("延时2秒报错：${e.printStackTrace()}");
                e.printStackTrace();
            }
            if (!ScreenUtil.isScreenOn(WeworkController.weworkService)) {
                LogUtil.e("判断当前屏幕是黑屏状态，重新唤醒一下");
                ScreenUtil.wakeAndUnlock(WeworkController.weworkService, true);
            }
            for (AccessibilityWindowInfo window : WeworkController.weworkService.getWindows()) {
                AccessibilityNodeInfo root = window.getRoot();
                if (root != null && TextUtils.isEmpty(root.getPackageName())) {
                    if (root.getPackageName().equals(Constant.PACKAGE_NAMES)) {

                    }
                } else {

                }
            }
        }
        return null;
    }

    /**
     * @author lkj
     * @date 2024/4/25 10:11
     * @desc 选择一个身份进入到主界面
     */
    public void toStatus(TaskBean taskBean) {
        //需要选择工作身份时 选择其中一个进入主界面
        LogUtil.e("选择工作身份");
        ArrayList<AccessibilityNodeInfo> nodeInfos = OutGroupCustomReturn.getInstance().getNodeList(taskBean, Views.TextView);
        if (!nodeInfos.isEmpty() && nodeInfos.size() > 0) {
            for (AccessibilityNodeInfo nodeInfo : nodeInfos) {
                if (!TextUtils.isEmpty(nodeInfo.getText())) {
                    Rect rect = new Rect();
                    nodeInfo.getBoundsInScreen(rect);
                    LogUtil.e("找的所有的TextView：" + rect.left + "  " + nodeInfo.getText().toString());
                    if (rect.left == 32 && !nodeInfo.getText().toString().equals("选择身份进入")) {
                        AccessibilityUtil.INSTANCE.performClick(nodeInfo, true);
                        LogUtil.e("点击一个身份进入: " + nodeInfo.getText().toString());
                        SystemClock.sleep(NumberUtils.getRandom1000() - 1000);
                        boolean into = OutGroupCustomReturn.getInstance().clickButtonText(taskBean, "进入", null);
                        break;
                    }
                }
            }
        } else {
            LogUtil.e("textViews 为空了");
            LogUtil.e("任务标识 全部设置 false");
            GroupSendImplKt.setFalse();
        }
    }

    /**
     * @author lkj
     * @date 2024/4/25 9:52
     * @desc 上滑挂断电话的手势
     */
    public void hangUpCall() {
        AccessibilityUtil.INSTANCE.scrollByXY(WeworkController.weworkService, 360, 250, 0, 0);
        AccessibilityUtil.INSTANCE.scrollByXY_sec(WeworkController.weworkService, 360, 250, 0, -250);
    }
}
