package com.kuxuan.autoclick.service;

import android.accessibilityservice.AccessibilityService;
import android.annotation.SuppressLint;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Rect;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.os.PowerManager;
import android.os.SystemClock;
import android.text.TextUtils;
import android.view.accessibility.AccessibilityEvent;
import android.view.accessibility.AccessibilityNodeInfo;
import android.view.accessibility.AccessibilityWindowInfo;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.kuxuan.autoclick.model.EventBean;
import com.kuxuan.autoclick.model.LeaderCreateBroupBean;
import com.kuxuan.autoclick.model.MainBean;
import com.kuxuan.autoclick.model.TaskBean;
import com.kuxuan.autoclick.model.groupsend.CompanyBean;
import com.kuxuan.autoclick.model.Constant;
import com.kuxuan.autoclick.model.EventNotifyBean;
import com.kuxuan.autoclick.net.GroupLeaderCreateGroupNet;
import com.kuxuan.autoclick.ui.MainActivity;
import com.kuxuan.autoclick.utils.AccessibilityUtil;
import com.kuxuan.autoclick.utils.NumberUtils;
import com.kuxuan.autoclick.utils.SPUtils;
import com.kuxuan.autoclick.utils.ScreenUtil;
import com.kuxuan.autoclick.utils.StringUtils;
import com.kuxuan.autoclick.utils.Views;
import com.kuxuan.autoclick.utils.LogUtil;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.*;

import static com.kuxuan.autoclick.service.WeworkController.weworkService;

/**
 * @Auth lkj
 * @Date 2023/5/26 18:22
 * @Describe 无障碍模式服务
 */
public class MyAccessibilityService extends AccessibilityService {
    private PowerManager.WakeLock wakeLock = null;
    public String currentPackage = "";
    public String currentClass = "";
    public List<CompanyBean> companyBeans;
    public Context context;
    public List<String> testNameList = new ArrayList<>();

    /**
     * @auth lkj
     * @Date 2023/5/31 16:40
     * @Describe 可用企业列表
     */
    public List<CompanyBean> canUserCompanyNameList = new ArrayList<>();
    public List<CompanyBean> nameList = new ArrayList<>();
    private GroupSendThread groupSendThread;
    private TaskBean taskBean;
    private String flagClassName = "";
    private Timer timer;
    public String needCreateCompanyName;//需要创建群聊的主体名
    public List<LeaderCreateBroupBean.RstBean.GroupBeansBean> groupBeans;//需要创建的群聊名列表

    @SuppressLint("NewApi")
    @Override
    public void onCreate() {
        super.onCreate();
        /**
         * @auth lkj
         * @Date 2023/6/12 14:44
         * @Describe 设置在息屏状态下  服务也保持运行状态
         */
        EventBus.getDefault().register(this);
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, MyAccessibilityService.class.getName());
        wakeLock.acquire();
        context = getBaseContext();
        /* 注册屏幕唤醒时的广播 */
        IntentFilter mScreenOnFilter = new IntentFilter("android.intent.action.SCREEN_ON");
        this.registerReceiver(mScreenOReceiver, mScreenOnFilter);

        /* 注册机器锁屏时的广播 */
        IntentFilter mScreenOffFilter = new IntentFilter("android.intent.action.SCREEN_OFF");
        this.registerReceiver(mScreenOReceiver, mScreenOffFilter);

    }

    private BroadcastReceiver mScreenOReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (action.equals("android.intent.action.SCREEN_ON")) {
                LogUtil.e("—— 屏幕亮了 ——");
            } else if (action.equals("android.intent.action.SCREEN_OFF")) {
                LogUtil.e("—— 屏幕黑了 ——");
            }
        }
    };

    @Override
    public void onDestroy() {
        if (wakeLock != null) {
            wakeLock.release();
            wakeLock = null;
        }
        super.onDestroy();
        EventBus.getDefault().unregister(this);
        this.unregisterReceiver(mScreenOReceiver);
    }

    @Override
    public void onAccessibilityEvent(AccessibilityEvent event) {
        if (event.getClassName() == null) {
            LogUtil.e("event.getClassName  为空了");
            return;
        }
//        com.android.systemui  充电的界面会显示
//        LogUtil.e("当前页面: " + " className: " + event.getClassName());
        if (event.getPackageName() == null) {
            LogUtil.e("event.getPackageName 为空了");
            return;
        }
        if (event.getPackageName().equals("android")) {
            LogUtil.e("一个单词的包名 不刷新");
            return;
        }
//        LogUtil.e("当前页面: " + " PackageName: " + event.getPackageName() + " ------- className: " + event.getClassName());
        if (event.getClassName().toString().equals(flagClassName)) {
//            LogUtil.e("类名相同  只输出一个");
            return;
        }
        flagClassName = event.getClassName().toString();
        if (event.getClassName().toString().contains("android.widget.") || event.getClassName().toString().contains("androidx.")) {
//            LogUtil.e("android 控件名输出");
            return;
        }
        String[] strings = new String[1];
        strings[0] = Views.TextView;
        AccessibilityNodeInfo root = getRootInActiveWindow();
        for (AccessibilityWindowInfo window : weworkService.getWindows()) {
            if (window.getRoot() != null) {
                root = window.getRoot();
            } else {
                LogUtil.e("根节点检测： 检测为空");
            }
        }
        ArrayList<AccessibilityNodeInfo> allByClazz = AccessibilityUtil.INSTANCE.findAllByClazz(root, strings);
        StringBuilder builder = new StringBuilder();
        int count = 0;
        for (AccessibilityNodeInfo byClazz : allByClazz) {
            if (byClazz.getText() != null) {
                count++;
                builder.append(count + ". " + byClazz.getText().toString() + "  ");
            }
        }
        if (event.getClassName().toString().contains("Dialog")) {
            LogUtil.e("有 弹框出现 不需要更新窗口节点 " + event.getClassName().toString());
            LogUtil.e("弹框下的文字： " + builder.toString());
            if (builder.toString().contains("快速会议")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("取消")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 取消 btn");
                    }
                }
            } else if (builder.toString().contains("打卡提醒")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("暂不")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 暂不 btn");
                    }
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("关闭")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 关闭 btn");
                    }
                }
            } else if (builder.toString().contains("企业微信没有响应")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("确定")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 确定 btn");
                    }
                }
            } else if (builder.toString().contains("群发工具没有响应")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("确定")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 确定 btn");
                        SystemClock.sleep(2000);
                        Intent intent = new Intent();
                        intent.setClassName("com.kuxuan.autoclick", "com.kuxuan.autoclick.ui.MainActivity");
                        startActivity(intent);
                    }
                }
            } else if (builder.toString().contains("邀请你语音通话") || builder.toString().contains("邀请你视频通话")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    Rect rect = new Rect();
                    byClazz.getBoundsInScreen(rect);
                    //[108,1422][252,1566]
                    if ((rect.left >= 106 && rect.left <= 110) && (rect.top >= 1420 && rect.top <= 1434) && (rect.right >= 250 && rect.right <= 254) && (rect.bottom >= 1564 && rect.bottom <= 1568)) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 挂断 按钮");
                    }
                }
            } else if (builder.toString().contains("日志上传提示")) {
                for (AccessibilityNodeInfo byClazz : allByClazz) {
                    if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("上传")) {
                        AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                        LogUtil.e("点击了 日志 上传 btn");
                    }
                }
            }
            return;
        } else {
            LogUtil.e("界面信息文字： " + builder.toString());
        }
//        LogUtil.e("当前包: " + event.getPackageName());
        currentPackage = event.getPackageName().toString();
        String className = event.getClassName().toString();
        if (className.contains(currentPackage)) {
            LogUtil.e("更新当前页面: currentPackage: " + event.getPackageName() + " className: " + event.getClassName());
            currentClass = event.getClassName().toString();
            EventBus.getDefault().postSticky(new EventBean());
        } else {
            LogUtil.e("不在企业微信中的信息： " + builder.toString());
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void getWindowInfo(EventBean num) {
        String[] strings = new String[2];
        strings[0] = Views.TextView;
        strings[1] = "android.widget.Button";
        strings[1] = "android.widget.ImageView";
        AccessibilityNodeInfo root = getRootInActiveWindow();
        for (AccessibilityWindowInfo window : weworkService.getWindows()) {
            if (window.getRoot() != null) {
                root = window.getRoot();
            } else {
                LogUtil.e("根节点检测： 检测为空");
            }
        }
        ArrayList<AccessibilityNodeInfo> allByClazz = AccessibilityUtil.INSTANCE.findAllByClazz(root, strings);
        StringBuilder builder = new StringBuilder();
        int count = 0;
        for (AccessibilityNodeInfo byClazz : allByClazz) {
            if (byClazz.getText() != null) {
                count++;
                builder.append(count + ". " + byClazz.getText().toString() + "  ");
                Rect rect = new Rect();
                byClazz.getBoundsInScreen(rect);
//                LogUtil.e(rect.toShortString() + " -- " + byClazz.getText().toString());
            }
        }
        LogUtil.e("企业微信中的信息： " + builder.toString());
        if (builder.toString().contains("企业微信没有响应")) {
            for (AccessibilityNodeInfo byClazz : allByClazz) {
                if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("确定")) {
                    AccessibilityUtil.INSTANCE.performClick(byClazz, true);
                    LogUtil.e("点击了 确定 btn");
                }
            }
        }
        if (builder.toString().contains("日志上传提示")) {
            for (AccessibilityNodeInfo byClazz : allByClazz) {
                if (!TextUtils.isEmpty(byClazz.getText()) && byClazz.getText().toString().equals("上传")) {
                    AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                    LogUtil.e("点击了 日志 上传 btn");
                }
            }
        }
        if (builder.toString().contains("邀请你语音通话") || builder.toString().contains("邀请你视频通话")) {
            for (AccessibilityNodeInfo byClazz : allByClazz) {
                Rect rect = new Rect();
                byClazz.getBoundsInScreen(rect);
                //[108,1422][252,1566]
                if ((rect.left >= 106 && rect.left <= 110) && (rect.top >= 1420 && rect.top <= 1434) && (rect.right >= 250 && rect.right <= 254) && (rect.bottom >= 1564 && rect.bottom <= 1568)) {
                    AccessibilityUtil.INSTANCE.clickByNodeCopy(this, byClazz);
                    LogUtil.e("点击了 挂断 按钮");
                }
            }
        }
    }

    /**
     * @author lkj
     * @date 2024/3/8 16:55
     * @desc 启动APP的方法
     */
    public void startApp() {
        Intent thisIntent = new Intent(context, MainActivity.class);
        thisIntent.setAction("android.intent.action.MAIN");
        thisIntent.addCategory("android.intent.category.LAUNCHER");
        thisIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(thisIntent);
        LogUtil.e("执行重新启动APP了！");
    }

    @Override
    public void onInterrupt() {
        LogUtil.e("onInterrupt!!");
    }

    @SuppressLint("NewApi")
    @Override
    protected void onServiceConnected() {
        super.onServiceConnected();
        WeworkController.weworkService = this;
    }

    /**
     * @auth lkj
     * @Date 2023/5/29 15:14
     * @Describe 从mainActivity中得到通知消息
     */
    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetMessage(EventNotifyBean bean) {
        initThread();
    }

    private void initThread() {
        /**
         * @auth lkj
         * @Date 2023/5/26 18:19
         * @Describe 20分钟发起一次网络请求
         */
        if (groupSendThread == null) {
            groupSendThread = new GroupSendThread();
            groupSendThread.start();

        } else {
            LogUtil.e("当前还在执行任务中 " + Constant.sendGroupIsRuning + "  " + Constant.addGroupIsRuning + "  " + Constant.friendSendIsRuning + "  " + Constant.isShareToColleague + "  " + Constant.isAddShareClient + "  " + Constant.isLeaderCreateGroup + "  " + Constant.isOutGroupCustomReturn + "  " + Constant.isRealTimeAddGroup);
            if (Constant.sendGroupIsRuning || Constant.addGroupIsRuning || Constant.friendSendIsRuning || Constant.isShareToColleague || Constant.isAddShareClient || Constant.isLeaderCreateGroup || Constant.isOutGroupCustomReturn || Constant.isRealTimeAddGroup) {
            } else {
                groupSendThread.againStart();
            }
        }
    }

    public List<TaskBean> taskList = new ArrayList<>();

    private TaskBean setTaskListStatus() {
        TaskBean taskBean = new TaskBean();
        taskBean.setId(System.currentTimeMillis());
        taskBean.setLive(true);
        LogUtil.e("显示 taskList 数量 " + taskList.size());
        if (taskList.size() > 0) {
            for (TaskBean bean : taskList) {
                bean.setLive(false);
                Constant.sendGroupIsRuning = false;
                Constant.addGroupIsRuning = false;
                Constant.friendSendIsRuning = false;
                Constant.isShareToColleague = false;
                Constant.isAddShareClient = false;
                Constant.isLeaderCreateGroup = false;
                Constant.isOutGroupCustomReturn = false;
                Constant.isRealTimeAddGroup = false;
            }
            taskList.clear();
            taskList.add(taskBean);
        } else {
            taskList.add(taskBean);
        }
        return taskBean;
    }

    /**
     * @auth lkj
     * @Date 2023/10/17 10:20
     * @Describe list 去重复
     */
    @RequiresApi(api = Build.VERSION_CODES.N)
    public List<CompanyBean> remDup() {
        String comName = "";
        List<CompanyBean> companyBeanList = new ArrayList<>();
        for (CompanyBean companyBean : nameList) {
            if (!companyBean.getCompanyName().equals(comName)) {
                comName = companyBean.getCompanyName();
                LogUtil.e("去重后的集合：" + companyBean.getCompanyName());
                companyBeanList.add(companyBean);
            }
        }
        return companyBeanList;
    }

    /**
     * @auth lkj
     * @Date 2023/6/12 10:26
     * @Describe 群发消息子线程
     */
    private class GroupSendThread extends Thread {
        private boolean isAgain = false;

        public GroupSendThread() {
            taskBean = setTaskListStatus();
            SelectMethod.getInstance().selectMethod1(taskBean);
            startCreateGroup(taskBean);
            //添加功能  给同事共享客户
//            shareToColleague(taskBean);
        }

        @Override
        public void run() {
            while (!isInterrupted()) {
                SystemClock.sleep(1000 * 60 * 20);
                if (!isAgain) {
                    if (Constant.sendGroupIsRuning || Constant.addGroupIsRuning || Constant.friendSendIsRuning || Constant.isShareToColleague || Constant.isAddShareClient || Constant.isLeaderCreateGroup || Constant.isOutGroupCustomReturn || Constant.isRealTimeAddGroup) {
                        LogUtil.e("当前还在执行任务中，请点击“重新启动”按钮启动群发工具");
                        StringUtils.setMessageData("当前还在执行任务中，请点击“重新启动”按钮启动群发工具", 20, 0);
                    } else {
                        //如果选择了邀请入群 这时将循环创建群组的功能开启
                        taskBean = setTaskListStatus();
                        SelectMethod.getInstance().selectMethod1(taskBean);
                    }
                } else {
                    isAgain = false;
                    LogUtil.e("这里解除了again操作");
                }
            }
        }

        public void againStart() {
            LogUtil.e("这里执行了again操作");
            isAgain = true;
            TaskBean taskBean = setTaskListStatus();
            SelectMethod.getInstance().selectMethod1(taskBean);
        }
    }

    //群主自动创建群 任务
    private void createGroupThread(TaskBean taskBean) {
        if (timer == null) {
            timer = new Timer();
            TimerTask timerTask = new TimerTask() {
                @Override
                public void run() {
                    // 定时器做的任务放在这里
                    GroupLeaderCreateGroupNet.getInstance().groupLeader(taskBean);
                }
            };
            timer.schedule(timerTask, NumberUtils.getRandom1000(), 30 * 1000);
        }
    }

    private void startCreateGroup(TaskBean taskBean) {
        boolean invite = SPUtils.getInstance().getBoolean("invite", true);
        if (invite) {
            createGroupThread(taskBean);
        } else {
            if (timer != null) {
                timer.cancel();
            }
        }
    }

    public void startOtherTask(boolean isJump, TaskBean taskBean) {
        /**
         * @auth lkj
         * @Date 2023/5/30 18:23
         * @Describe 判断群发流程是否执行完
         */
        if (!isJump) {
            if (canUserCompanyNameList == null || canUserCompanyNameList.size() == 0 || Constant.listIndex >= canUserCompanyNameList.size()) {
                Constant.listIndex = 0;
                //当群发消息的任务执行完毕 将执行标签置位false
                Constant.sendGroupIsRuning = false;
                LogUtil.e("群发消息执行任务结束");
                return;
            }
        }
        //点亮屏幕的方法(正常任务会在这里判断)
        if (taskBean.isLive()) {
            weakUpScreen(taskBean);
        } else {
            LogUtil.e("taskLive: " + taskBean.isLive());
            // 执行的 群主创建群任务时会走这里
            if (taskBean.isCreateGroup()) {
                LogUtil.e("走进了群主建群功能");
                weakUpScreen(taskBean);
            } else {
                LogUtil.e("isCreateGroup: " + taskBean.isCreateGroup());
            }
        }
    }

    /**
     * @param taskBean
     * @auth lkj
     * @Date 2023/6/16 17:28
     * @Describe 唤醒屏幕
     */
    private void weakUpScreen(TaskBean taskBean) {
        if (ScreenUtil.isScreenOn(this)) {
            LogUtil.e("屏幕正亮着呢");
            ScreenUtil.openScreen(taskBean);
        } else {
            LogUtil.e("屏幕没亮，唤醒一下");
            new Thread(() -> {
                try {
                    ScreenUtil.wakeAndUnlock(MyAccessibilityService.this, true);
                    Thread.sleep(2000);
                    Message message = Message.obtain();
                    message.what = 100;
                    message.obj = taskBean;
                    handler.sendMessage(message);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }).start();
        }
    }

    private Handler handler = new Handler(new Handler.Callback() {
        @Override
        public boolean handleMessage(@NonNull Message message) {
            if (message.what == 100) {
                TaskBean taskBean = (TaskBean) message.obj;
                weakUpScreen(taskBean);
            }
            return false;
        }
    });

    public void runStop() {
        EventBus.getDefault().post(new MainBean(false));
        StringUtils.setMessageData("", 10, 0);
        StringUtils.setMessageData("", 20, 0);
    }
    /**
     *  邀请入群流程
     *  1.客户端每N分钟请求一次服务端接口，接口返回请求参数中的开始时间和结束时间范围内新添加的客户信息[N为5~10分钟以内的随机数，开始时间和结束时间的时间间隔不超过10分钟]
     *  2.根据服务端接口详情结果，匹配与本设备相关的企微数据，按企微分组数据后，切换企微
     *  3.查找客户群[按客户群名称]，进入指定群聊后获取群人数，超过限制人数自动切换到新群
     *  4.按客户昵称进行搜索、拉群
     *  待确认1.添加客服后立即删除，是否可以拉入群
     */

    /**
     * 功能排期：
     * 接口对接、调试接口、两次请求避免冲突（可考虑使用锁在执行任务时，若需要请求服务器时，则此时跳过不请求服务器）      --> 6.6
     * 切换企微（使用现有逻辑，是否可行）、列表中查找群（进入后获取人数并加入判断需要加入的人数）人满自动换新群、拉人入群 --> 6.7  6.8
     * 测试整体功能                                                                                          --> 6.9
     * ***注意：群不需要模拟创建***
     * 处理被删除后的人，在执行拉人入群时会产生弹框，将弹框处理掉(已处理)
     */

    /**
     * 目前需要再增加 朋友圈群发的功能
     * 群发消息单次执行15条
     * 朋友圈群发 任务执行流程
     */

    /**
     * @Author: lkj
     * @Date: 2023/7/13 10:15
     * @Desc: 备注：情况1：当请求平台成功，数据列表为空时，群发助手只会进入到企微主页，之后不会执行其他操作。
     *             情况2：当请求平台成功，数据列表有数据，但没有企业符合企微主体时，群发助手会到主体界面检查一遍返回到企微主页，之后不会执行其他操作
     *
     */
    /**
     * @author lkj
     * @date 2024/3/20 10:00
     * @desc 使用七麦关键词分析工具，重新编辑OPPO vivo应用是现场的应用简介和副标题，覆盖排名靠前的关键词
     */
}
