package com.kuxuan.autoclick.utils;

import java.util.List;
import java.util.Locale;

import android.R.string;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.telephony.TelephonyManager;
import android.util.Log;

/**
 * @auth lkj
 * @Date 2023/12/20 13:54
 * @Describe
 */
public class DeviceInfoUtils {

    /**
     * 获取设备宽度（px）
     */
    public static int getDeviceWidth(Context context) {
        return context.getResources().getDisplayMetrics().widthPixels;
    }

    /**
     * 获取设备高度（px）
     */
    public static int getDeviceHeight(Context context) {
        return context.getResources().getDisplayMetrics().heightPixels;
    }

    /**
     * 获取设备的唯一标识， 需要 “android.permission.READ_Phone_STATE”权限
     */
    public static String getIMEI(Context context) {
        TelephonyManager tm = (TelephonyManager) context
                .getSystemService(Context.TELEPHONY_SERVICE);
        String deviceId = tm.getDeviceId();
        if (deviceId == null) {
            return "UnKnown";
        } else {
            return deviceId;
        }
    }

    /**
     * 获取厂商名
     **/
    public static String getDeviceManufacturer() {
        return android.os.Build.MANUFACTURER;
    }

    /**
     * 获取产品名
     **/
    public static String getDeviceProduct() {
        return android.os.Build.PRODUCT;
    }

    /**
     * 获取手机品牌
     */
    public static String getDeviceBrand() {
        return android.os.Build.BRAND;
    }

    /**
     * 获取手机型号
     */
    public static String getDeviceModel() {
        return android.os.Build.MODEL;
    }

    /**
     * 获取手机主板名
     */
    public static String getDeviceBoard() {
        return android.os.Build.BOARD;
    }

    /**
     * 设备名
     **/
    public static String getDeviceDevice() {
        return android.os.Build.DEVICE;
    }

    /**
     * fingerprit 信息
     **/
    public static String getDeviceFubgerprint() {
        return android.os.Build.FINGERPRINT;
    }

    /**
     * 硬件名
     **/
    public static String getDeviceHardware() {
        return android.os.Build.HARDWARE;
    }

    /**
     * 主机
     **/
    public static String getDeviceHost() {
        return android.os.Build.HOST;
    }

    /**
     * 显示ID
     **/
    public static String getDeviceDisplay() {
        return android.os.Build.DISPLAY;
    }

    /**
     * ID
     **/
    public static String getDeviceId() {
        return android.os.Build.ID;
    }

    /**
     * 获取手机用户名
     **/
    public static String getDeviceUser() {
        return android.os.Build.USER;
    }

    /**
     * 获取手机 硬件序列号
     **/
    public static String getDeviceSerial() {
        return android.os.Build.SERIAL;
    }

    /**
     * 获取手机Android 系统SDK
     *
     * @return
     */
    public static int getDeviceSDK() {
        return android.os.Build.VERSION.SDK_INT;
    }

    /**
     * 获取手机Android 版本
     *
     * @return
     */
    public static String getDeviceAndroidVersion() {
        return android.os.Build.VERSION.RELEASE;
    }

    /**
     * 获取当前手机系统语言。
     */
    public static String getDeviceDefaultLanguage() {
        return Locale.getDefault().getLanguage();
    }

    /**
     * 获取当前系统上的语言列表(Locale列表)
     */
    public static String getDeviceSupportLanguage() {
        Log.e("wangjie", "Local: " + Locale.GERMAN);
        Log.e("wangjie", "Local: " + Locale.ENGLISH);
        Log.e("wangjie", "Local: " + Locale.US);
        Log.e("wangjie", "Local: " + Locale.CHINESE);
        Log.e("wangjie", "Local: " + Locale.TAIWAN);
        Log.e("wangjie", "Local: " + Locale.FRANCE);
        Log.e("wangjie", "Local: " + Locale.FRENCH);
        Log.e("wangjie", "Local: " + Locale.GERMANY);
        Log.e("wangjie", "Local: " + Locale.ITALIAN);
        Log.e("wangjie", "Local: " + Locale.JAPAN);
        Log.e("wangjie", "Local: " + Locale.JAPANESE);
        return Locale.getAvailableLocales().toString();
    }

    public static String getDeviceAllInfo(Context context) {

        return
                "\n1. 手机系统: " + CustomOSUtils.getCustomOS("XIAOMI") + "  " + CustomOSUtils.getCustomOSVersion("XIAOMI")

                        + "\n2. 设备宽度: " + getDeviceWidth(context)

                        + "\n3. 设备高度: " + getDeviceHeight(context)

                        + "\n4. 是否有内置SD卡: " + SDCardUtils.isSDCardMount()

                        + "\n5. RAM 信息: " + SDCardUtils.getRAMInfo(context)

//                        + "\n6. 内部存储信息" + SDCardUtils.getStorageInfo(context, 0)

//                        + "\n7. SD卡 信息: " + SDCardUtils.getStorageInfo(context, 1)

//                + "\n8. 是否联网: " + Utils.isNetworkConnected(context)

//                + "\n9. 网络类型: " + Utils.GetNetworkType(context)

                        + "\n10. 系统默认语言: " + getDeviceDefaultLanguage()

                        + "\n11. 硬件序列号(设备名): " + android.os.Build.SERIAL

                        + "\n12. 手机型号: " + android.os.Build.MODEL

                        + "\n13. 生产厂商: " + android.os.Build.MANUFACTURER

                        + "\n14. 手机Fingerprint标识: " + android.os.Build.FINGERPRINT

                        + "\n15. Android 版本: " + android.os.Build.VERSION.RELEASE

                        + "\n16. Android SDK版本: " + android.os.Build.VERSION.SDK_INT

                        + "\n17. 安全patch 时间: " + android.os.Build.VERSION.SECURITY_PATCH

//                + "\n18. 发布时间: " + Utils.Utc2Local(android.os.Build.TIME)

                        + "\n19. 版本类型: " + android.os.Build.TYPE

                        + "\n20. 用户名: " + android.os.Build.USER

                        + "\n21. 产品名: " + android.os.Build.PRODUCT

                        + "\n22. ID: " + android.os.Build.ID

                        + "\n23. 显示ID: " + android.os.Build.DISPLAY

                        + "\n24. 硬件名: " + android.os.Build.HARDWARE

                        + "\n25. 产品名: " + android.os.Build.DEVICE

                        + "\n26. Bootloader: " + android.os.Build.BOOTLOADER

                        + "\n27. 主板名: " + android.os.Build.BOARD

                        + "\n28. CodeName: " + android.os.Build.VERSION.CODENAME
                        + "\n29. 语言支持: " + getDeviceSupportLanguage();

    }

    /**
     * @author lkj
     * @date 2024/2/21 10:38
     * @desc 获取企业微信版本号
     */
    public static void getTencentVersion(Context context) {
        PackageManager packageManager = context.getPackageManager();
        List<PackageInfo> packageInfos = packageManager.getInstalledPackages(0);
        for (PackageInfo packageInfo : packageInfos) {
            // 获取应用的包名
            String packageName = packageInfo.packageName;
            // 获取应用的版本号
            int versionCode = packageInfo.versionCode;
            String versionName = packageInfo.versionName;
            // 根据包名判断是否为目标应用
            if (packageName.equals("com.tencent.wework")) {
                // 处理目标应用的版本号
            }
            LogUtil.e("packageName: " + packageName);
//            LogUtil.e("Version code: " + versionCode);
//            LogUtil.e("Version name: " + versionName);
        }
    }
}