package com.kuxuan.autoclick.utils;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.util.Log;

import com.kuxuan.autoclick.app.Mapp;
import com.kuxuan.autoclick.model.MainBean;
import com.kuxuan.autoclick.service.WeworkController;

import org.greenrobot.eventbus.EventBus;

public class LogUtil {
    //当前使用log的类名
    private static String className;
    // 当前使用log的方法名
    private static String methodName;
    //当前使用log的行数
    private static int lineNumber;
    //log的开关,在Application中设置，上线关闭，测试打开
    private static boolean debuggable = true;
    private Context mContext;
    private static LogUtil INSTANCE = new LogUtil();
    private static String versionName;

    private LogUtil() {
    }

    public static LogUtil getInstance() {
        return INSTANCE;
    }

    /**
     * 初始化
     *
     * @param context
     */
    public void init(Context context) {
        mContext = context;
        PackageManager pManager = context.getPackageManager();
        try {
            PackageInfo pInfo = pManager.getPackageInfo(context.getPackageName(), PackageManager.GET_ACTIVITIES);
            versionName = pInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
    }

    public static void setDebuggable(boolean debuggable) {
        LogUtil.debuggable = debuggable;
    }

    private static boolean isDebuggable() {
        return debuggable;
    }

    //设置显示的log信息
    private static String createLog(String log) {
        StringBuffer stringBuffer = new StringBuffer();
        String memPercent = MemoruyUtils.memoryUsageRate(Mapp.context) + "%";
        stringBuffer.append("=====").append(methodName).append("(").append(className).append(":").append(lineNumber).append(")==" + versionName + " 内存：" + memPercent + "==").append(log);
        LogFileUtils.writeLogFile(stringBuffer.toString());
        return stringBuffer.toString();
    }

    //根据获取到的堆栈跟踪信息，得到当前调用这个类的类名、方法名以及行数
    private static void getName(StackTraceElement[] stackTraceElements) {
        className = stackTraceElements[1].getFileName();
        methodName = stackTraceElements[1].getMethodName();
        lineNumber = stackTraceElements[1].getLineNumber();
    }

    public static void e(String msg) {
        if (!isDebuggable()) {
            return;
        }
//        StringUtils.setMessageData(msg, 30);
        //获取堆栈信息
        getName(new Throwable().getStackTrace());
        Log.e(className, createLog(msg));
    }

    public static void d(String msg) {
        if (!isDebuggable()) {
            return;
        }
        getName(new Throwable().getStackTrace());
        Log.d(className, createLog(msg));
    }

    public static void i(String msg) {
        if (!isDebuggable()) {
            return;
        }
        getName(new Throwable().getStackTrace());
        Log.i(className, createLog(msg));
    }

    public static void w(String msg) {
        if (!isDebuggable()) {
            return;
        }
        getName(new Throwable().getStackTrace());
        Log.w(className, createLog(msg));
    }

}