package com.kuxuan.autoclick.utils;

import android.app.ActivityManager;
import android.content.Context;
import android.os.Build;
import android.os.Debug;
import android.os.SystemClock;
import android.os.storage.StorageManager;
import android.os.storage.StorageVolume;

import androidx.annotation.RequiresApi;

import com.kuxuan.autoclick.app.Mapp;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.List;


/**
 * @author lkj
 * @date 2024/2/23 10:11
 * @desc 内存获取
 */
public class MemoruyUtils {
    /**
     * @author lkj
     * @date 2024/2/23 10:22
     * @desc 获取总内存大小
     */
    public static long getTotalMemory(Context context) {
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        ActivityManager.MemoryInfo memoryInfo = new ActivityManager.MemoryInfo();
        activityManager.getMemoryInfo(memoryInfo);
        return memoryInfo.totalMem;
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:22
     * @desc 获取可用内存大小
     */
    public static long getAvailableMemory(Context context) {
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        ActivityManager.MemoryInfo memoryInfo = new ActivityManager.MemoryInfo();
        activityManager.getMemoryInfo(memoryInfo);
        return memoryInfo.availMem;
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:22
     * @desc 获取应用占用内存大小
     */
    public static long getApplicationMemory() {
        Debug.MemoryInfo memoryInfo = new Debug.MemoryInfo();
        Debug.getMemoryInfo(memoryInfo);
        return memoryInfo.getTotalPss();
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:54
     * @desc 当前内存使用率
     */
    public static int memoryUsageRate(Context context) {
        long totalMemory = MemoruyUtils.getTotalMemory(context);
        long availableMemory = MemoruyUtils.getAvailableMemory(context);
        int percent = (int) ((totalMemory - availableMemory) / (float) totalMemory * 100);
        return percent;
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:25
     * @desc 内部存储 总大小
     */
    public static String getInternalStorageSize(Context context) {
        File filesDir = context.getFilesDir();
        return storageSizeConversion(filesDir.getTotalSpace());
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:25
     * @desc 内部存储 可用空间大小
     */
    public static String getFreeSpace(Context context) {
        File filesDir = context.getFilesDir();
        return storageSizeConversion(filesDir.getFreeSpace());
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:28
     * @desc 扩展TF卡 总大小
     */
    @RequiresApi(api = Build.VERSION_CODES.M)
    public static long getExternalStorageSize(Context context) {
        StorageManager storageManager = context.getSystemService(StorageManager.class);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            for (StorageVolume volume : storageManager.getStorageVolumes()) {
                if (volume.isRemovable()) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                        File directory = volume.getDirectory();
                        LogUtil.e("directory = " + directory.getAbsolutePath());
                        LogUtil.e("directory = " + directory.getTotalSpace());
                        return directory.getTotalSpace();
                    }
                } else {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                        File directory = volume.getDirectory();
                        LogUtil.e("false directory = " + directory.getAbsolutePath());
                        LogUtil.e("false directory = " + directory.getTotalSpace());
                    }
                }
            }
        }
        return 0;
    }

    /**
     * @author lkj
     * @date 2024/2/23 10:29
     * @desc 扩展TF卡 可用空间大小
     */
    @RequiresApi(api = Build.VERSION_CODES.M)
    public static long getExternalStorageFreeSize(Context context) {
        StorageManager storageManager = context.getSystemService(StorageManager.class);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            for (StorageVolume volume : storageManager.getStorageVolumes()) {
                if (volume.isRemovable()) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                        File directory = volume.getDirectory();
                        LogUtil.e("directory = " + directory.getAbsolutePath());
                        LogUtil.e("free = " + directory.getFreeSpace());
                        return directory.getFreeSpace();
                    }
                } else {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                        File directory = volume.getDirectory();
                        LogUtil.e("false directory = " + directory.getAbsolutePath());
                        LogUtil.e("false directory = " + directory.getTotalSpace());
                    }
                }
            }
        }
        return 0;
    }

    /**
     * @param size
     * @author lkj
     * @date 2024/2/23 10:24
     * @desc 单位转换
     */
    public static String storageSizeConversion(double size) {
        DecimalFormat df = new DecimalFormat("#0.0");
        String fileSizeString = "";
        if (size < 1024) {
            fileSizeString = df.format((double) size) + "B";
        } else if (size < 1048576) {
            fileSizeString = df.format((double) size / 1024) + "K";
        } else if (size < 1073741824) {
            fileSizeString = df.format((double) size / 1048576) + "M";
        } else {
            fileSizeString = df.format((double) size / 1073741824) + "G";
        }
        return fileSizeString;
    }

    /**
     * 清理内存
     *
     * @param context
     */
    public static void clearMemory(Context context) {
        cleanCache(context);
        ActivityManager activityManger = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> list = activityManger.getRunningAppProcesses();
        if (list != null) {
            for (int i = 0; i < list.size(); i++) {
                ActivityManager.RunningAppProcessInfo apinfo = list.get(i);
                String[] pkgList = apinfo.pkgList;
                if (apinfo.importance > ActivityManager.RunningAppProcessInfo.IMPORTANCE_VISIBLE) {
                    for (int j = 0; j < pkgList.length; j++) {
                        /**清理不可用的内容空间**/
                        activityManger.killBackgroundProcesses(pkgList[j]);
                    }
                }
            }
        }
        List<ActivityManager.RunningAppProcessInfo> processInfos = activityManger.getRunningAppProcesses();
        if (processInfos != null) {
            for (int i = 0; i < processInfos.size(); i++) {
                ActivityManager.RunningAppProcessInfo apinfo = processInfos.get(i);
                LogUtil.e("pid            " + apinfo.pid);
                LogUtil.e("processName              " + apinfo.processName);
                LogUtil.e("importance            " + apinfo.importance);
                String[] pkgList = apinfo.pkgList;

                if (apinfo.importance > ActivityManager.RunningAppProcessInfo.IMPORTANCE_SERVICE) {
                    // Process.killProcess(apinfo.pid);
                    for (int j = 0; j < pkgList.length; j++) {
                        //2.2以上是过时的,请用killBackgroundProcesses代替
                        activityManger.killBackgroundProcesses(pkgList[j]);
                    }
                }
            }
        }

    }

    /**
     * 清理应用缓存
     *
     * @param mContext
     */
    public static void cleanCache(Context mContext) {
        File cacheDir = mContext.getCacheDir();
        if (cacheDir != null && cacheDir.isDirectory()) {
            File[] files = cacheDir.listFiles();
            if (files != null) {
                for (File file : files) {
                    file.delete();
                }
            }
        }
    }

    private static final String TAG = "CPUUsage";

    public static float getCPUUsage(String pkgName) {
        try {
            int pid = android.os.Process.myPid();
            String path = "/proc/" + pid + "/stat";
            BufferedReader reader = new BufferedReader(new FileReader(path));
            String line = reader.readLine();
            reader.close();
            String[] tokens = line.split(" ");
            long utime = Long.parseLong(tokens[13]);
            long stime = Long.parseLong(tokens[14]);
            long cutime = Long.parseLong(tokens[15]);
            long cstime = Long.parseLong(tokens[16]);
            long starttime = Long.parseLong(tokens[21]);
            long totalTime = utime + stime + cutime + cstime;
            long elapsedTime = SystemClock.elapsedRealtime() * 1000;
            float cpuUsage = (totalTime / (float) (elapsedTime - starttime)) * 100;
            LogUtil.e("CPU usage: " + cpuUsage);
            return cpuUsage;
        } catch (IOException e) {
            LogUtil.e("Error getting CPU usage  " + e);
            return 0;
        }
    }

    /**
     * @author lkj
     * @date 2024/3/11 11:39
     * @desc app的内存实时使用情况
     */
    public static long getMemoryUsage(String pkgName) {
        try {
            ActivityManager activityManager = (ActivityManager) Mapp.context.getSystemService(Context.ACTIVITY_SERVICE);
            List<ActivityManager.RunningAppProcessInfo> runningAppProcesses = activityManager.getRunningAppProcesses();
            for (ActivityManager.RunningAppProcessInfo runningAppProcess : runningAppProcesses) {
                LogUtil.e("运行的APP：" + runningAppProcess.processName);
            }
            Debug.MemoryInfo memoryInfo = new Debug.MemoryInfo();
            Debug.getMemoryInfo(memoryInfo);
            int[] pids = new int[10];
            pids[0] = android.os.Process.myPid();
            Debug.MemoryInfo[] memoryInfos = activityManager.getProcessMemoryInfo(pids);
            long memoryUsage = memoryInfos[0].getTotalPss() * 1024; // getTotalPss()返回值单位是KB，需要转换成字节
            LogUtil.e("Memory usage of " + pkgName + " is " + storageSizeConversion(memoryUsage) + " bytes");
            return memoryUsage;
        } catch (Exception e) {
            LogUtil.e("Failed to get memory usage " + e);
            return 0;
        }
    }
}
/**
 * 车厘子：
 * 爱汀课网络文化传播：      15:50 发送数量 1
 * 众课五常分公司：          15:53 发送数量 13
 * 众课传播余杭分公司：      15:59 发送数量 7
 */



