package com.kuxuan.autoclick.utils;

import android.annotation.SuppressLint;
import android.app.KeyguardManager;
import android.content.Context;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;

import com.kuxuan.autoclick.model.TaskBean;
import com.kuxuan.autoclick.service.GroupSendImplKt;

import static android.content.ContentValues.TAG;
import static android.content.Context.KEYGUARD_SERVICE;
import static android.content.Context.POWER_SERVICE;

/**
 * @Auth lkj
 * @Date 2023/5/26 14:09
 * @Describe Android判断当前应用程序状态是在前台还是后台
 */
public class ScreenUtil {
    private static WakeLock wakeLock;

    @SuppressLint("InvalidWakeLockTag")
    public static void keepScreenOn(Context context, boolean on) {
        if (on) {
            PowerManager pm = (PowerManager) context.getSystemService(POWER_SERVICE);
            wakeLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ON_AFTER_RELEASE, TAG);
            wakeLock.acquire();
        } else {
            if (wakeLock != null) {
                wakeLock.release();
                wakeLock = null;
            }
        }
    }

    /**
     * @auth lkj
     * @Date 2023/5/26 16:38
     * @Describe 判断屏幕是否亮着
     */
    public static boolean isScreenOn(Context context) {
        PowerManager powerManager = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
        //true为打开，false为关闭
        boolean ifOpen = powerManager.isScreenOn();
        return ifOpen;
    }


    /**
     * @auth lkj
     * @Date 2023/5/26 16:45
     * @Describe 屏幕唤醒
     */
    @SuppressLint("InvalidWakeLockTag")
    public static void wakeUp(Context context) {
        PowerManager powerManager = (PowerManager) context.getSystemService(POWER_SERVICE);
        LogUtil.d("wjz debug wakeUp: screen is on ? == " + powerManager.isInteractive());
        if (!powerManager.isInteractive()) {
            //屏锁管理器
//            KeyguardManager km = (KeyguardManager) getSystemService(Context.KEYGUARD_SERVICE);
//            KeyguardManager.KeyguardLock kl = km.newKeyguardLock("unLock");
//            //解锁
//            kl.disableKeyguard();
            WakeLock wakeLock = powerManager.newWakeLock(PowerManager.ACQUIRE_CAUSES_WAKEUP | PowerManager.SCREEN_DIM_WAKE_LOCK, "cyg_wake");

            //点亮屏幕
            wakeLock.acquire(10000);
            //释放
            wakeLock.release();
        }
    }

    private static KeyguardManager km;
    private static KeyguardManager.KeyguardLock kl;
    private static PowerManager pm;
    private static WakeLock wl;

    @SuppressLint({"InvalidWakeLockTag", "MissingPermission"})
    public static void wakeAndUnlock(Context context, boolean b) {
        if (b) {
            //获取电源管理器对象
            pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
            //获取PowerManager.WakeLock对象，后面的参数|表示同时传入两个值，最后的是调试用的Tag
            wl = pm.newWakeLock(PowerManager.ACQUIRE_CAUSES_WAKEUP | PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "bright");
            //点亮屏幕
            wl.acquire(10000);
            //得到键盘锁管理器对象
            km = (KeyguardManager) context.getSystemService(KEYGUARD_SERVICE);
            kl = km.newKeyguardLock("unLock");
            //解锁
            kl.disableKeyguard();
        } else {
            //锁屏
            kl.reenableKeyguard();
            //释放wakeLock，关灯
            wl.release();
        }
    }

    /**
     * @auth lkj
     * @Date 2023/5/29 17:30
     * @Describe 判断当前屏幕是否锁屏
     */
    public static boolean isLockScreen(Context context) {
        KeyguardManager mKeyguardManager = (KeyguardManager) context.getSystemService(KEYGUARD_SERVICE);
        LogUtil.e("当前的锁屏状态： " + mKeyguardManager.inKeyguardRestrictedInputMode());
        return mKeyguardManager.inKeyguardRestrictedInputMode();
//        if (WeworkController.weworkService.currentPackage.equals(Constant.SYSTEM_UI) && WeworkController.weworkService.currentClass.equals(Constant.LOCK_SCREEN_UI)) {
//            return true;
//        } else {
//            return false;
//        }
    }
    /**
     * @auth lkj
     * @Date 2023/5/30 15:20
     * @Describe 滑动解锁
     */
    public static void openScreen(TaskBean taskBean) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                LogUtil.e("屏幕没锁");
                GroupSendImplKt.isAtApp(taskBean);
            }
        }).start();
    }
}
