package com.kuxuan.autoclick.utils;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.graphics.Rect;
import android.os.Build;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewTreeObserver;

import java.nio.charset.StandardCharsets;
import java.util.UUID;


/**
 * @Auth lkj
 * @Date 2023/5/29 11:06
 * @Describe 获得一部Android设备的唯一标识码
 * 业务需求：
 * 1、同一部手机，多次获取、卸载重装，得到的设备标识是同一个值
 * 2、不同手机，获得的设备标识一定要不同
 * <p>
 * 解决方案：
 * 使用AndroidId + 硬件信息生成的UUID
 * 1、获得硬件UUID（无需权限，根据硬件相关属性，生成UUID）
 * 2、获取AndroidId（无需权限，极个别设备获取不到数据，拿不到时，使用的是Random）
 * 3、使用AndroidId和硬件UUID生成一个设备唯一标识UUID，
 * 4、对设备唯一标识UUID做32位的MD5加密，得到一个固定长度的序列号
 */
public class SystemUtils {

    public static String getDeviceId(Context context) {
        final int targetSdkVersion = context.getApplicationInfo().targetSdkVersion;
        if (targetSdkVersion > Build.VERSION_CODES.P && Build.VERSION.SDK_INT > Build.VERSION_CODES.P) {
            return getUniqueID(context);
        } else {
            return getTelId(context);
        }
    }

    @SuppressLint({"HardwareIds", "MissingPermission"})
    private static String getTelId(Context context) {
        final TelephonyManager manager = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
        return manager.getDeviceId();
    }

    private static String getUniqueID(Context context) {
        String id = null;
        @SuppressLint("HardwareIds") final String androidId = Settings.Secure.getString(context.getContentResolver(), Settings.Secure.ANDROID_ID);
        if (!TextUtils.isEmpty(androidId) && !"9774d56d682e549c".equals(androidId)) {
            UUID uuid = UUID.nameUUIDFromBytes(androidId.getBytes(StandardCharsets.UTF_8));
            id = uuid.toString();
        }

        if (TextUtils.isEmpty(id)) {
            id = getUUID();
        }

        return TextUtils.isEmpty(id) ? UUID.randomUUID().toString() : id;
    }
    @SuppressLint({"NewApi", "MissingPermission"})
    private static String getUUID() {
        String serial = null;

         String m_szDevIDShort = "35" +
                Build.BOARD.length() % 10 + Build.BRAND.length() % 10 +

                ((null != Build.SUPPORTED_ABIS) ? Build.SUPPORTED_ABIS.length : 0) % 10 +

                Build.DEVICE.length() % 10 + Build.DISPLAY.length() % 10 +

                Build.HOST.length() % 10 + Build.ID.length() % 10 +

                Build.MANUFACTURER.length() % 10 + Build.MODEL.length() % 10 +

                Build.PRODUCT.length() % 10 + Build.TAGS.length() % 10 +

                Build.TYPE.length() % 10 + Build.USER.length() % 10; //13 位

        if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.P) {
            try {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    serial = android.os.Build.getSerial();
                } else {
                    serial = Build.SERIAL;
                }
                //API>=9 使用serial号
                return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
            } catch (Exception exception) {
                serial = "serial" + UUID.randomUUID().toString(); // 随便一个初始化
            }
        } else {
            serial = android.os.Build.UNKNOWN + UUID.randomUUID().toString(); // 随便一个初始化
        }

        //使用硬件信息拼凑出来的15位号码
        return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
    }
}
