package com.kuxuan.autoclick.utils.get_expection;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Environment;

import com.kuxuan.autoclick.model.Constant;
import com.kuxuan.autoclick.net.callback.ResultCallBack;
import com.kuxuan.autoclick.net.request.AccessPresenter;
import com.kuxuan.autoclick.service.GroupSendImplKt;
import com.kuxuan.autoclick.utils.SPUtils;
import com.kuxuan.autoclick.utils.StringSort;
import com.kuxuan.autoclick.utils.DateUtil;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.SystemUtils;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 异常处理类，继承UncaughtException处理类
 * <p>
 * Created by wk
 */
public class MyCrashHandler implements Thread.UncaughtExceptionHandler {

    private Thread.UncaughtExceptionHandler crashHandler;
    private Context mContext;
    // CrashHandler实例
    private static MyCrashHandler INSTANCE = new MyCrashHandler();
    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 单例模式,保证只有一个CrashHandler实例
     */
    private MyCrashHandler() {
    }

    public static MyCrashHandler getInstance() {
        return INSTANCE;
    }

    /**
     * 初始化
     *
     * @param context
     */
    public void init(Context context) {
        mContext = context;
        // 获取UncaughtException处理器
        crashHandler = Thread.getDefaultUncaughtExceptionHandler();
        Thread.setDefaultUncaughtExceptionHandler(this);
    }

    /**
     * 重写方法，异常发生时调用
     */
    @Override
    public void uncaughtException(Thread thread, Throwable ex) {
        if (!dealException(ex) && crashHandler != null) {
            crashHandler.uncaughtException(thread, ex);
        }
        //此处还可以进行一些异常后续处理，如杀死进程，finish页面的操作
    }

    /**
     * 收集设备参数信息
     *
     * @param context
     */
    public Map<String, String> collectDeviceInfo(Context context) {
        Map<String, String> infos = new HashMap<String, String>();
        try {
            PackageManager pManager = context.getPackageManager();
            PackageInfo pInfo = pManager.getPackageInfo(context.getPackageName(), PackageManager.GET_ACTIVITIES);
            if (pInfo != null) {
                String versionName = "";
                if (versionName == null) {
                    versionName = null;
                } else {
                    versionName = pInfo.versionName;
                }
                String versionCode = pInfo.versionCode + "";
                infos.put("versionName", versionName);
                infos.put("versionCode", versionCode);
            }
        } catch (PackageManager.NameNotFoundException e) {

        }
        //获取手机的信息
        Field[] fields = Build.class.getDeclaredFields();
        for (Field field : fields) {
            try {
                //跳过安全检查，Filed是private的话会抛出异常，以防万一
                field.setAccessible(true);
                infos.put(field.getName(), field.get(null).toString());
            } catch (Exception e) {
            }
        }
        return infos;
    }

    /**
     * 保存错误信息到文件中
     *
     * @param ex
     * @return 返回文件名称, 便于将文件传送到服务器
     */
    public void saveCrashInfo2File(Throwable ex, Map<String, String> infos) {

        StringBuffer sb = new StringBuffer();
        //遍历文件
        for (Map.Entry<String, String> entry : infos.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            sb.append(key + "=" + value + "\n");
        }
//        WeworkController.INSTANCE.getWeworkService()
        Writer writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        ex.printStackTrace(printWriter);
        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        printWriter.close();
        String result = writer.toString();
        sb.append(result);
//        postReport(sb.toString());
        if (result.contains("java.net.SocketTimeoutException")
                || result.contains("java.net.UnknownHostException")
                || result.contains("java.net.ConnectException")
                || result.contains("java.net.SocketException")
                || result.contains(" android.view.WindowManager$BadTokenException")
        ) {
            return;
        }
        postReport(result);
        try {
            long timestamp = System.currentTimeMillis();
            //记录当前时间
            String time = DateUtil.stampToDate(timestamp);
            String fileName = "crash-" + time + "-" + timestamp + ".txt";
            LogUtil.e("异常上报：fileName： " + fileName);
            if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
                //保存路径
                String path = "/storage/emulated/0/Android/media/com.kuxuan.autoclick/crash/";
                LogUtil.e("异常上报：path： " + path);
                // 获取应用内部文件目录路径
//                File directory = new File(path);
//                if (!directory.exists()) {
//                    directory.mkdir();
//                }
//                // 创建文件对象，参数为文件路径和文件名
//                File file = new File(directory, "example.txt");
//                // 打开文件输出流
//                FileOutputStream outputStream = new FileOutputStream(file);
//                // 要写入的数据
//                String data = sb.toString();
//                // 将数据写入文件
//                outputStream.write(data.getBytes());
//                // 关闭文件输出流
//                outputStream.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 自定义错误处理,收集错误信息 发送错误报告等操作均在此完成.
     *
     * @param ex
     * @return true:如果处理了该异常信息;否则返回false.
     */
    public boolean dealException(Throwable ex) {
        if (ex == null) {
            return false;
        }
        // 收集设备参数信息
        Map<String, String> infos = collectDeviceInfo(mContext);
        // 保存日志文件
        ex.printStackTrace();
        LogUtil.e("全局检测到异常了" + ex.getMessage());
        saveCrashInfo2File(ex, infos);
        return true;
    }

    // 使用HTTP Post 发送错误报告到服务器
    // 在上传的时候还可以将该app的version，该手机的机型等信息一并发送的服务器，
    // Android的兼容性众所周知，所以可能错误不是每个手机都会报错，还是有针对性的去debug比较好
    private void postReport(String exception) {
        String currCompany = SPUtils.getInstance().getString("currCompany");
        String currUser = SPUtils.getInstance().getString("currUser");
        String msg = DateUtil.getDateTime() + " " + currUser + " " + currCompany + " " + exception;
        String device_id = SystemUtils.getDeviceId(mContext);
        Map<String, Object> map = new HashMap<>();
        map.put("device_id", device_id);
        map.put("exception", msg);
        map.put("version", getAppVersionName(mContext));
        String sign = StringSort.sortKey(map);
//        String requestUrl = Constant.http_debug + Constant.warnReport;
        String requestUrl = Constant.http_release + Constant.warnReport;
        LogUtil.e("请求链接：" + requestUrl);
        LogUtil.e("任务标识 全部设置 false");
        GroupSendImplKt.setFalse();
        AccessPresenter.getInstance().warnReport(device_id, format.format(new Date()) + "   " + msg, getAppVersionName(mContext), sign, new ResultCallBack<String>() {
            @Override
            public void success(String result) {
                LogUtil.e("异常上报后返回：" + result);
            }

            @Override
            public void fail(String error) {
                LogUtil.e("异常上报后返回：" + error);
            }

            @Override
            public void requestComplete() {

            }
        });
    }

    /**
     * @auth lkj
     * @Date 2023/6/9 16:52
     * @Describe 获取App版本号
     */
    public static String getAppVersionName(Context context) {
        String versionName = "";
        try {
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
            versionName = pi.versionName;
            if (versionName == null || versionName.length() <= 0) {
                return "";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return versionName;
    }
}