package com.kuxuan.autoclick.view;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.OpenableColumns;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.kuxuan.autoclick.R;
import com.kuxuan.autoclick.model.Constant;
import com.kuxuan.autoclick.ui.MainActivity;
import com.kuxuan.autoclick.utils.DecodeUttils;
import com.kuxuan.autoclick.utils.LogUtil;
import com.kuxuan.autoclick.utils.MemoruyUtils;
import com.kuxuan.autoclick.utils.SPUtils;
import com.kuxuan.autoclick.utils.SystemUtils;
import com.zx.uploadlibrary.listener.ProgressListener;
import com.zx.uploadlibrary.utils.OKHttpUtils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Response;

/**
 * @author lkj
 * @date 2024/2/27 16:57
 * @desc 自定义 提示dialog
 */
public class CommitDialog extends Dialog {
    @BindView(R.id.user_name_input)
    EditText userNameInput;
    @BindView(R.id.company_name_input)
    EditText companyNameInput;
    @BindView(R.id.bug_type_btn2)
    CheckBox bugTypeBtn2;
    @BindView(R.id.bug_type_btn3)
    CheckBox bugTypeBtn3;
    @BindView(R.id.time_stemp)
    Spinner timeStemp;
    @BindView(R.id.bug_desc_input)
    EditText bugDescInput;
    @BindView(R.id.let_me_want)
    TextView letMeWant;
    @BindView(R.id.upload_log)
    Button uploadLog;
    @BindView(R.id.file_name_list)
    RecyclerView fileNameList;
    @BindView(R.id.upload_progress)
    ProgressBar uploadProgress;
    @BindView(R.id.progress_tv)
    TextView progressTv;
    @BindView(R.id.progress_layout)
    LinearLayout progressLayout;
    @BindView(R.id.bug_type_btn1)
    CheckBox bugTypeBtn1;
    @BindView(R.id.manager_input)
    EditText managerInput;
    @BindView(R.id.identity_group_leader)
    RadioButton identityGroupLeader;
    @BindView(R.id.identity_group_custom)
    RadioButton identityGroupCustom;
    @BindView(R.id.identity_group)
    RadioGroup identityGroup;
    @BindView(R.id.identity_layout)
    LinearLayout identityLayout;
    private Activity context;
    private OnClickListener onClickListener;
    private String[] spinners = new String[]{"请选择", "00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00", "11:00",
            "12:00", "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00", "23:00"};
    private List<String> fileNames = new ArrayList<>();
    private FileNameAdapter adapter;
    private List<String> stringList;

    private boolean upLoadSuccessed = false;
    private int progress;

    private static final int START = 100;
    private static final int PROGRESS = 101;
    private static final int FINISH = 102;

    public CommitDialog(Activity context, OnClickListener onClickListener) {
        super(context);
        this.context = context;
        this.onClickListener = onClickListener;
    }

    public CommitDialog(Activity context, boolean cancelable, OnCancelListener cancelListener, OnClickListener onClickListener) {
        super(context, cancelable, cancelListener);
        this.onClickListener = onClickListener;
        this.context = context;
    }

    public CommitDialog(Activity context, int themeResId, OnClickListener onClickListener) {
        super(context, themeResId);
        this.onClickListener = onClickListener;
        this.context = context;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.dialog_confirm);
        ButterKnife.bind(this);
        setCanceledOnTouchOutside(false);
        getWindow().setBackgroundDrawableResource(android.R.color.transparent);
        initSpinner();
        initDate();
        initAdapter();
        handler.sendEmptyMessageDelayed(10, 2000);
    }

    @SuppressLint("HandlerLeak")
    private Handler handler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case START:
                    progressLayout.setVisibility(View.VISIBLE);
                    letMeWant.setEnabled(false);
                    letMeWant.setClickable(false);
                    uploadLog.setEnabled(false);
                    uploadLog.setClickable(false);
                    break;
                case PROGRESS:
                    uploadProgress.setProgress(progress);
                    progressTv.setText(progress + "%");
                    break;
                case FINISH:
                    uiSetTrue(true);
                    break;
                default:
                    break;
            }
        }
    };

    private void initDate() {
        String manager = SPUtils.getInstance().getString("manager");
        String user_name = SPUtils.getInstance().getString("user_name");
        String company_name = SPUtils.getInstance().getString("company_name");
        boolean bug_type1 = SPUtils.getInstance().getBoolean("bug_type1", false);
        boolean bug_type2 = SPUtils.getInstance().getBoolean("bug_type2", false);
        boolean bug_type3 = SPUtils.getInstance().getBoolean("bug_type3", false);
        int time_stemp = SPUtils.getInstance().getInt("time_stemp");
        String bug_desc = SPUtils.getInstance().getString("bug_desc");
        LogUtil.e("check: " + bug_type1 + "  " + bug_type2 + "  " + bug_type3);
        if (bug_type3) {
            identityLayout.setVisibility(View.VISIBLE);
        } else {
            identityLayout.setVisibility(View.GONE);
        }
        int identity_group = SPUtils.getInstance().getInt("identity_group", -1);
        if (identity_group == R.id.identity_group_leader) {//群主
            identityGroupLeader.setChecked(true);
        }
        if (identity_group == R.id.identity_group_custom) {//客服
            identityGroupCustom.setChecked(true);
        }
        managerInput.setText(manager);
        userNameInput.setText(user_name);
        companyNameInput.setText(company_name);
        bugTypeBtn1.setChecked(bug_type1);
        bugTypeBtn2.setChecked(bug_type2);
        bugTypeBtn3.setChecked(bug_type3);
        timeStemp.setSelection(time_stemp);
        bugDescInput.setText(bug_desc);
        identityGroup.setOnCheckedChangeListener((group, checkedId) -> {
            SPUtils.getInstance().put("identity_group", checkedId);
        });
        bugTypeBtn3.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                identityLayout.setVisibility(View.VISIBLE);
            } else {
                identityLayout.setVisibility(View.GONE);
            }
        });
    }

    private void initAdapter() {
        stringList = new ArrayList<>();
        stringList.add("+点击添加日志文件");
        LinearLayoutManager manager = new LinearLayoutManager(context);
        manager.setOrientation(RecyclerView.VERTICAL);
        adapter = new FileNameAdapter(stringList);
        fileNameList.setLayoutManager(manager);
        fileNameList.setAdapter(adapter);
    }

    private void initSpinner() {
        ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(context, R.layout.spinner_item, spinners);
        timeStemp.setAdapter(arrayAdapter);
    }

    public void setContent(Uri uri) {
        Cursor cursor = context.getContentResolver().query(uri, null, null, null, null);
        if (cursor != null) {
            int nameIndex = cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME);
            cursor.moveToFirst();
            //文件名
            String file_name = cursor.getString(nameIndex);
            cursor.close();
            LogUtil.e("这个路径：  " + Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MOVIES) + File.separator + "kobe.mp4");
            String fileName = uri.getPath().replace("/document/primary:", "/storage/emulated/0/");
            if (!fileNames.contains(fileName)) {
                fileNames.add(fileName);
            } else {
                LogUtil.e("该文件已选择: " + file_name);
                Toast.makeText(context, "该文件已选择！", Toast.LENGTH_SHORT).show();
                return;
            }
            if (adapter != null) {
                stringList.add(0, file_name);
                adapter.upDateList(stringList);
            }
        }
    }

    @OnClick({R.id.let_me_want, R.id.upload_log})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.let_me_want:
                getDate(false);
                break;
            case R.id.upload_log:
                if (upLoadSuccessed) {
                    dismiss();
                } else {
                    getDate(true);
                }
                break;
        }
    }

    private void jumpFileSelector() {
//        String path = "Android/media/com.kuxuan.autoclick/Log日志";
        String path = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MOVIES) + File.separator;
//        Uri uri = Uri.parse("content://com.android.externalstorage.documents/document/primary:" + path);
        Uri uri = Uri.parse(path);
        // 创建Intent对象，指定Action为ACTION_GET_CONTENT
        Intent logIntent = new Intent(Intent.ACTION_GET_CONTENT);
        // 设置Intent的类型为文件夹
//                logIntent.setType("*/*");
//                // 设置Intent的标识为选择文件夹
//                logIntent.addCategory(Intent.CATEGORY_OPENABLE);
        //进入指定文件夹中
        logIntent.setDataAndType(uri, "*/*");
        // 启动文件管理器
        context.startActivityForResult(logIntent, MainActivity.REQUEST_CODE);
    }

    private boolean isFirst = false;

    private void getDate(boolean isUpLoad) {
        String manager = managerInput.getText().toString();
        String userName = userNameInput.getText().toString();
        String companyName = companyNameInput.getText().toString();
        boolean checked1 = bugTypeBtn1.isChecked();
        boolean checked2 = bugTypeBtn2.isChecked();
        boolean checked3 = bugTypeBtn3.isChecked();
        int itemPosition = timeStemp.getSelectedItemPosition();
//        String file_name = fileName.getText().toString();
        String bugDesc = bugDescInput.getText().toString();
        LogUtil.e("check: " + checked1 + "  " + checked2 + "  " + checked3);
        SPUtils.getInstance().put("manager", manager);
        SPUtils.getInstance().put("user_name", userName);
        SPUtils.getInstance().put("company_name", companyName);
        SPUtils.getInstance().put("bug_type1", checked1);
        SPUtils.getInstance().put("bug_type2", checked2);
        SPUtils.getInstance().put("bug_type3", checked3);
        SPUtils.getInstance().put("time_stemp", itemPosition);
        SPUtils.getInstance().put("bug_desc", bugDesc);
        //让输入框失去焦点，然后关闭软键盘
        InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(userNameInput.getWindowToken(), 0);
        userNameInput.clearFocus();
        imm.hideSoftInputFromWindow(companyNameInput.getWindowToken(), 0);
        companyNameInput.clearFocus();
        imm.hideSoftInputFromWindow(bugDescInput.getWindowToken(), 0);
        bugDescInput.clearFocus();
        if (isUpLoad) {//是不是点击的上传按钮
            if (TextUtils.isEmpty(userName)) {
                Toast.makeText(context, "请输入客服名称", Toast.LENGTH_SHORT).show();
                return;
            }
            if (TextUtils.isEmpty(companyName)) {
                Toast.makeText(context, "请输入主体名称", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!checked1 && !checked2 && !checked3) {
                Toast.makeText(context, "请选择最少一个问题类型", Toast.LENGTH_SHORT).show();
                return;
            }
            if (itemPosition == 0) {
                Toast.makeText(context, "请选择时间节点", Toast.LENGTH_SHORT).show();
                return;
            }
            if (TextUtils.isEmpty(bugDesc)) {
                Toast.makeText(context, "请输入问题描述", Toast.LENGTH_SHORT).show();
                return;
            }
            if (stringList.size() <= 1) {
                Toast.makeText(context, "请添加日志文件", Toast.LENGTH_SHORT).show();
                return;
            }
            StringBuilder msg_type = new StringBuilder();
            if (checked1) {
                msg_type.append(bugTypeBtn1.getText().toString() + ",");
            }
            if (checked2) {
                msg_type.append(bugTypeBtn2.getText().toString() + ",");
            }
            int identity_group = SPUtils.getInstance().getInt("identity_group", -1);
            if (checked3) {
                msg_type.append(bugTypeBtn3.getText().toString() + ",");
                if (identity_group == -1) {
                    Toast.makeText(context, "请选择当前身份", Toast.LENGTH_SHORT).show();
                    return;
                }
            }
            String manager_leader_name = SPUtils.getInstance().getString("manager_leader_name");
            String identity = "";
            if (identity_group == R.id.identity_group_leader) {//群主
                identity = "群主";
            }
            if (identity_group == R.id.identity_group_custom) {//客服
                identity = "客服 - 群主是：" + manager_leader_name;
            }
            String deviceId = SystemUtils.getDeviceId(context);
            List<File> files = new ArrayList<>();
            HashMap<String, String> map = new HashMap<>();
            map.put("user_name", userName + " - " + manager);
            map.put("corp_name", companyName);
            map.put("subject", userName + "-" + manager + "-" + companyName + "-" + spinners[itemPosition]);
            map.put("time_interval", spinners[itemPosition]);
            map.put("msg_type", msg_type.toString() + " - " + identity);
            map.put("description", bugDesc);
            map.put("device_id", deviceId);
            for (String fileName : fileNames) {
                File file = new File(fileName);
                files.add(file);
            }
            ProgressListener uiProgressRequestListener = new ProgressListener() {
                @Override
                public void onProgress(long currentBytes, long contentLength, boolean done) {
                    if (!isFirst) {
                        isFirst = true;
                        handler.sendEmptyMessage(START);
                    }
                    progress = (int) ((100 * currentBytes) / contentLength);
//                    LogUtil.e("文件上传进度：" + progress + "％");
                    handler.sendEmptyMessage(PROGRESS);
                    if (done) {
                        handler.sendEmptyMessage(FINISH);
                    }
                }
//                @Override
//                public void onUIProgress(long bytesWrite, long contentLength, boolean done) {
////                    LogUtil.e("bytesWrite:" + bytesWrite);
////                    LogUtil.e("contentLength" + contentLength);
////                    LogUtil.e((100 * bytesWrite) / contentLength + " % done ");
//                    LogUtil.e("================================");
//                    //ui层回调,设置当前上传的进度值
//                    progress = (int) ((100 * bytesWrite) / contentLength);
//                    LogUtil.e("文件上传进度：" + progress + "％");
//                    progressTv.setText(progress + "%");
//                    if (progress == 100) {
//                        uiSetTrue(true);
//                    }
//                    handler.sendEmptyMessage(100);
//                }

                //上传开始
//                @Override
//                public void onUIStart(long bytesWrite, long contentLength, boolean done) {
//                    super.onUIStart(bytesWrite, contentLength, done);
//                    progressLayout.setVisibility(View.VISIBLE);
//                    letMeWant.setEnabled(false);
//                    letMeWant.setClickable(false);
//                    uploadLog.setEnabled(false);
//                    uploadLog.setClickable(false);
//                }
//
//                //上传结束
//                @Override
//                public void onUIFinish(long bytesWrite, long contentLength, boolean done) {
//                    super.onUIFinish(bytesWrite, contentLength, done);
//                    //uploadProgress.setVisibility(View.GONE); //设置进度条不可见
//                    Toast.makeText(context, "上传成功", Toast.LENGTH_SHORT).show();
//                    LogUtil.e("日志上传成功！");
//                    uiSetTrue(true);
//                }
            };
            for (int i = 0; i < fileNames.size(); i++) {
                File file = new File(fileNames.get(i));
                if (!file.exists()) {
                    // 文件不存在，进行处理
                    LogUtil.e("文件不存在：" + fileNames.get(i));
                } else {
                    LogUtil.e("文件存在：" + fileNames.get(i));
                }
            }
            new Thread(new Runnable() {
                @Override
                public void run() {
                    OKHttpUtils.doPostRequest(Constant.UPLOAD_LOG_URL, map, fileNames, uiProgressRequestListener, new Callback() {
                        @Override
                        public void onFailure(Call call, final IOException e) {
                            LogUtil.e("error------> " + e.getMessage());
                            e.printStackTrace();
                            context.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    uiSetTrue(false);
                                    Toast.makeText(context, "上传失败" + e.getMessage(), Toast.LENGTH_SHORT).show();
                                }
                            });
                        }

                        @Override
                        public void onResponse(Call call, Response response) throws IOException {
                            String result = DecodeUttils.decodeUnicode(response.body().string());
                            LogUtil.e("success---->" + result);
                        }
                    });
                }
            }).start();
        } else {
            dismiss();
        }
    }

    private void uiSetTrue(boolean success) {
        letMeWant.setEnabled(true);
        letMeWant.setClickable(true);
        uploadLog.setEnabled(true);
        uploadLog.setClickable(true);
        letMeWant.setVisibility(View.GONE);
        if (success) {
            uploadLog.setText("完成");
        } else {
            uploadLog.setText("重新上传");
        }
        upLoadSuccessed = success;
    }

    public interface OnClickListener {
        void onConfirm();
    }

    class FileNameAdapter extends RecyclerView.Adapter<FileNameAdapter.FileNameHolder> {
        private List<String> strings;

        public FileNameAdapter(List<String> strings) {
            this.strings = strings;
        }

        public void upDateList(List<String> strings) {
            this.strings = strings;
            notifyDataSetChanged();
        }

        @NonNull
        @Override
        public FileNameHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(context).inflate(R.layout.adapter_file_name, parent, false);
            FileNameHolder holder = new FileNameHolder(view);
            return holder;
        }

        @Override
        public void onBindViewHolder(@NonNull FileNameHolder holder, int position) {
            if (position < strings.size() - 1) {
                File file = new File(fileNames.get(position));
                long length = file.length();
                String sizeConversion = MemoruyUtils.storageSizeConversion(length);
                holder.fileName.setText(strings.get(position) + "(" + sizeConversion + ")");
                holder.fileName.setTextColor(context.getResources().getColor(R.color.black));
            } else {
                holder.fileName.setText(strings.get(position));
                holder.fileName.setTextColor(context.getResources().getColor(R.color.black));
            }
            if (position == strings.size() - 1) {
                holder.fileName.setTextColor(context.getResources().getColor(R.color.blue));
                holder.fileName.setOnClickListener(view -> {
                    jumpFileSelector();
                });
            }

        }

        @Override
        public int getItemCount() {
            return strings.size();
        }

        class FileNameHolder extends RecyclerView.ViewHolder {
            @BindView(R.id.file_name)
            TextView fileName;

            public FileNameHolder(@NonNull View itemView) {
                super(itemView);
                ButterKnife.bind(this, itemView);
            }
        }
    }
}
